@@ -1,34 +1,37 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
-# 
+# FOR A PARTICULAR PURPOSE.
+#
 ##############################################################################
 """ Customizable image objects that come from the filesystem.
 
-$Id: FSImage.py,v 1.14.18.3 2004/06/02 14:24:49 tseaver Exp $
+$Id: FSImage.py 37972 2005-08-16 20:54:10Z jens $
 """
 
-import string, os
-
 import Globals
 from DateTime import DateTime
 from AccessControl import ClassSecurityInfo
 from webdav.common import rfc1123_date
+from OFS.Cache import Cacheable
 from OFS.Image import Image, getImageInfo
 
-from utils import _dtmldir, _setCacheHeaders
-from CMFCorePermissions import ViewManagementScreens, View, FTPAccess
+from permissions import FTPAccess
+from permissions import View
+from permissions import ViewManagementScreens
+from DirectoryView import registerFileExtension
+from DirectoryView import registerMetaType
 from FSObject import FSObject
-from DirectoryView import registerFileExtension, registerMetaType, expandpath
+from utils import _dtmldir
+from utils import _setCacheHeaders, _ViewEmulator
+from utils import expandpath
 
-from OFS.Cache import Cacheable
 
 class FSImage(FSObject):
     """FSImages act like images but are not directly
@@ -73,15 +76,15 @@ class FSImage(FSObject):
         return data
 
     #### The following is mainly taken from OFS/Image.py ###
-        
-    __str__ = Image.__str__
 
-    _image_tag = Image.tag
+    __str__ = Image.__str__.im_func
+
+    _image_tag = Image.tag.im_func
     security.declareProtected(View, 'tag')
     def tag(self, *args, **kw):
         # Hook into an opportunity to reload metadata.
         self._updateFromFS()
-        return apply(self._image_tag, args, kw)
+        return self._image_tag(*args, **kw)
 
     security.declareProtected(View, 'index_html')
     def index_html(self, REQUEST, RESPONSE):
@@ -93,40 +96,48 @@ class FSImage(FSObject):
         """
         self._updateFromFS()
         data = self._data
+        data_len = len(data)
+        last_mod = self._file_mod_time
+        status = 200
         # HTTP If-Modified-Since header handling.
-        header=REQUEST.get_header('If-Modified-Since', None)
+        header = REQUEST.get_header('If-Modified-Since', None)
         if header is not None:
-            header=string.split(header, ';')[0]
+            header = header.split(';')[0]
             # Some proxies seem to send invalid date strings for this
             # header. If the date string is not valid, we ignore it
             # rather than raise an error to be generally consistent
             # with common servers such as Apache (which can usually
             # understand the screwy date string as a lucky side effect
             # of the way they parse it).
-            try:    mod_since=long(DateTime(header).timeTime())
-            except: mod_since=None
+            try:
+                mod_since = long(DateTime(header).timeTime())
+            except:
+                mod_since = None
+
             if mod_since is not None:
-                last_mod = self._file_mod_time
                 if last_mod > 0 and last_mod <= mod_since:
-                    # Set header values since apache caching will return
-                    # Content-Length of 0 in response if size is not set here
-                    RESPONSE.setHeader('Last-Modified', rfc1123_date(last_mod))
-                    RESPONSE.setHeader('Content-Type', self.content_type)
-                    RESPONSE.setHeader('Content-Length', len(data))
-                    RESPONSE.setStatus(304)
-                    return ''
+                    status = 304
+                    data = ''
 
-        #Last-Modified will get stomped on by a cache policy it there is one set....
-        RESPONSE.setHeader('Last-Modified', rfc1123_date(self._file_mod_time))
+        #Last-Modified will get stomped on by a cache policy it there is
+        #one set....
+        RESPONSE.setStatus(status)
+        RESPONSE.setHeader('Last-Modified', rfc1123_date(last_mod))
         RESPONSE.setHeader('Content-Type', self.content_type)
-        RESPONSE.setHeader('Content-Length', len(data))
 
-        #There are 2 Cache Managers which can be in play....need to decide which to use
-        #to determine where the cache headers are decided on.
+        if status != 304:
+            # Avoid setting content-length for a 304. See RFC 2616.
+            # Zope might still, for better or for worse, set a
+            # content-length header with value "0". 
+            RESPONSE.setHeader('Content-Length', data_len)
+
+        #There are 2 Cache Managers which can be in play....
+        #need to decide which to use to determine where the cache headers
+        #are decided on.
         if self.ZCacheable_getManager() is not None:
             self.ZCacheable_set(None)
         else:
-            _setCacheHeaders(self, extra_context={})
+            _setCacheHeaders(_ViewEmulator().__of__(self), extra_context={})
         return data
 
     security.declareProtected(View, 'getContentType')
