@@ -1,29 +1,34 @@
-# Copyright (c) 2001 New Information Paradigms Ltd
+##############################################################################
 #
-# This Software is released under the MIT License:
-# http://www.opensource.org/licenses/mit-license.html
-# See license.txt for more details.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
-# $Id: FSZSQLMethod.py,v 1.3.26.3 2004/04/23 21:11:33 sidnei Exp $
+# This software is subject to the provisions of the Zope Public License,
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
+# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
+# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
+# FOR A PARTICULAR PURPOSE.
+#
+##############################################################################
+""" Customizable ZSQL methods that come from the filesystem.
 
-""" (not yet)Customizable ZSQL methods that come from the filesystem.
-
-$Id: FSZSQLMethod.py,v 1.3.26.3 2004/04/23 21:11:33 sidnei Exp $
+$Id: FSZSQLMethod.py 38006 2005-08-19 15:06:40Z urbanape $
 """
 
 import Globals
 from AccessControl import ClassSecurityInfo
-from zLOG import LOG,ERROR
-
-from Products.CMFCore.CMFCorePermissions import View, ViewManagementScreens
-from Products.CMFCore.DirectoryView import registerFileExtension, registerMetaType, expandpath
-from Products.CMFCore.FSObject import FSObject
-
+from Acquisition import ImplicitAcquisitionWrapper
 from Products.ZSQLMethods.SQL import SQL
+from zLOG import LOG, ERROR
 
+from DirectoryView import registerFileExtension
+from DirectoryView import registerMetaType
+from FSObject import FSObject
+from permissions import View
+from permissions import ViewManagementScreens
 from utils import _dtmldir
+from utils import expandpath
 
-import Acquisition
 
 class FSZSQLMethod(SQL, FSObject):
     """FSZSQLMethods act like Z SQL Methods but are not directly
@@ -41,13 +46,13 @@ class FSZSQLMethod(SQL, FSObject):
 
     # Use declarative security
     security = ClassSecurityInfo()
-    
+
     security.declareObjectProtected(View)
-    
+
     # Make mutators private
     security.declarePrivate('manage_main','manage_edit','manage_advanced','manage_advancedForm')
     manage=None
-    
+
     security.declareProtected(ViewManagementScreens, 'manage_customise')
     manage_customise = Globals.DTMLFile('custzsql', _dtmldir)
 
@@ -66,8 +71,10 @@ class FSZSQLMethod(SQL, FSObject):
         s.manage_advanced(self.max_rows_,
                           self.max_cache_,
                           self.cache_time_,
-                          '',
-                          '')
+                          self.class_name_,
+                          self.class_file_,
+                          connection_hook=self.connection_hook,
+                          direct=self.allow_simple_one_argument_traversal)
         return s
 
     def _readFile(self, reparse):
@@ -90,28 +97,34 @@ class FSZSQLMethod(SQL, FSObject):
             if len(pair)!=2:
                 continue
             parameters[pair[0].strip().lower()]=pair[1].strip()
-
-        # check for required an optional parameters
-        try:            
-            title =         parameters.get('title','')
-            connection_id = parameters.get('connection id',parameters['connection_id'])
-            arguments =     parameters.get('arguments','')
-            max_rows =      parameters.get('max_rows',1000)
-            max_cache =     parameters.get('max_cache',100)
-            cache_time =    parameters.get('cache_time',0)            
+        
+        # check for required parameters
+        try:
+            connection_id =   ( parameters.get('connection id', '') or
+                                parameters['connection_id'] )
         except KeyError,e:
             raise ValueError,"The '%s' parameter is required but was not supplied" % e
-        
-        self.manage_edit(title,
-                         connection_id,
-                         arguments,
-                         template=data)
+
+        # Optional parameters
+        title =           parameters.get('title','')
+        arguments =       parameters.get('arguments','')
+        max_rows =        parameters.get('max_rows',1000)
+        max_cache =       parameters.get('max_cache',100)
+        cache_time =      parameters.get('cache_time',0)
+        class_name =      parameters.get('class_name','')
+        class_file =      parameters.get('class_file','')
+        connection_hook = parameters.get('connection_hook',None)
+        direct = parameters.get('allow_simple_one_argument_traversal', None)
+
+        self.manage_edit(title, connection_id, arguments, template=data)
 
         self.manage_advanced(max_rows,
                              max_cache,
                              cache_time,
-                             '', # don't really see any point in allowing
-                             '') # brain specification...
+                             class_name,
+                             class_file,
+                             connection_hook=connection_hook,
+                             direct=direct)
 
         # do we need to do anything on reparse?
 
@@ -120,11 +133,10 @@ class FSZSQLMethod(SQL, FSObject):
         # Provide an opportunity to update the properties.
         def __of__(self, parent):
             try:
-                self = Acquisition.ImplicitAcquisitionWrapper(self, parent)
+                self = ImplicitAcquisitionWrapper(self, parent)
                 self._updateFromFS()
                 return self
             except:
-                from zLOG import LOG, ERROR
                 import sys
                 LOG('FS Z SQL Method',
                     ERROR,
