@@ -3,39 +3,48 @@
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """ PortalFolder: CMF-enabled Folder objects.
 
-$Id: PortalFolder.py,v 1.45.4.7 2004/07/28 17:05:37 shh Exp $
+$Id: PortalFolder.py 40138 2005-11-15 17:47:37Z jens $
 """
 
-import sys
-import Globals, re, base64, marshal, string
+import base64
+import marshal
+import re
+from warnings import warn
 
-from CMFCorePermissions import View
-from CMFCorePermissions import ManageProperties
-from CMFCorePermissions import ListFolderContents
-from CMFCorePermissions import AddPortalFolders
-from CMFCorePermissions import AddPortalContent
-from CMFCatalogAware import CMFCatalogAware
-from OFS.Folder import Folder
-from OFS.ObjectManager import REPLACEABLE
-from Globals import DTMLFile
-from AccessControl import getSecurityManager, ClassSecurityInfo, Unauthorized
+from AccessControl import ClassSecurityInfo
+from AccessControl import getSecurityManager
 from Acquisition import aq_parent, aq_inner, aq_base
+from Globals import DTMLFile
+from Globals import InitializeClass
+from OFS.OrderSupport import OrderSupport
+from OFS.Folder import Folder
+
+from CMFCatalogAware import CMFCatalogAware
 from DynamicType import DynamicType
-from utils import getToolByName, _checkPermission
+from exceptions import AccessControl_Unauthorized
+from exceptions import BadRequest
+from exceptions import zExceptions_Unauthorized
+from interfaces.Folderish import Folderish as IFolderish
+from permissions import AddPortalContent
+from permissions import AddPortalFolders
+from permissions import ChangeLocalRoles
+from permissions import DeleteObjects
+from permissions import ListFolderContents
+from permissions import ManagePortal
+from permissions import ManageProperties
+from permissions import View
+from utils import _checkPermission
+from utils import getToolByName
 
-try:
-    from zExceptions import BadRequest
-except ImportError:
-    BadRequest = 'BadRequest'
 
 factory_type_information = (
   { 'id'             : 'Folder'
@@ -46,24 +55,41 @@ factory_type_information = (
   , 'factory'        : 'manage_addPortalFolder'
   , 'filter_content_types' : 0
   , 'immediate_view' : 'folder_edit_form'
+  , 'aliases'        : {'(Default)': 'index_html',
+                        'view': 'index_html',
+                        'index.html':'index_html'}
   , 'actions'        : ( { 'id'            : 'view'
                          , 'name'          : 'View'
                          , 'action': 'string:${object_url}'
                          , 'permissions'   : (View,)
-                         , 'category'      : 'folder'
                          }
                        , { 'id'            : 'edit'
                          , 'name'          : 'Edit'
                          , 'action': 'string:${object_url}/folder_edit_form'
                          , 'permissions'   : (ManageProperties,)
-                         , 'category'      : 'folder'
                          }
                        , { 'id'            : 'localroles'
                          , 'name'          : 'Local Roles'
                          , 'action':
                                   'string:${object_url}/folder_localrole_form'
-                         , 'permissions'   : (ManageProperties,)
-                         , 'category'      : 'folder'
+                         , 'permissions'   : (ChangeLocalRoles,)
+                         }
+                       , { 'id'            : 'folderContents'
+                         , 'name'          : 'Folder contents'
+                         , 'action': 'string:${object_url}/folder_contents'
+                         , 'permissions'   : (ListFolderContents,)
+                         }
+                       , { 'id'            : 'new'
+                         , 'name'          : 'New...'
+                         , 'action': 'string:${object_url}/folder_factories'
+                         , 'permissions'   : (AddPortalContent,)
+                         , 'visible'       : 0
+                         }
+                       , { 'id'            : 'rename_items'
+                         , 'name'          : 'Rename items'
+                         , 'action': 'string:${object_url}/folder_rename_form'
+                         , 'permissions'   : (AddPortalContent,)
+                         , 'visible'       : 0
                          }
                        )
   }
@@ -71,40 +97,43 @@ factory_type_information = (
 )
 
 
-class PortalFolder(DynamicType, CMFCatalogAware, Folder):
+class PortalFolderBase(DynamicType, CMFCatalogAware, Folder):
+    """Base class for portal folder
     """
-        Implements portal content management, but not UI details.
-    """
-    meta_type = 'Portal Folder'
-    portal_type = 'Folder'
+    meta_type = 'Portal Folder Base'
 
-    __implements__ = (DynamicType.__implements__, Folder.__implements__)
+    __implements__ = (IFolderish, DynamicType.__implements__,
+                      Folder.__implements__)
 
     security = ClassSecurityInfo()
 
     description = ''
 
-    manage_options = Folder.manage_options + \
-                     CMFCatalogAware.manage_options
+    manage_options = ( Folder.manage_options +
+                       CMFCatalogAware.manage_options )
 
     def __init__( self, id, title='' ):
         self.id = id
         self.title = title
 
+    #
+    #   'MutableDublinCore' interface methods
+    #
     security.declareProtected(ManageProperties, 'setTitle')
     def setTitle( self, title ):
-        """
-            Edit the folder title.
+        """ Set Dublin Core Title element - resource name.
         """
         self.title = title
 
     security.declareProtected(ManageProperties, 'setDescription')
     def setDescription( self, description ):
-        """
-            Edit the folder description.
+        """ Set Dublin Core Description element - resource summary.
         """
         self.description = description
 
+    #
+    #   other methods
+    #
     security.declareProtected(ManageProperties, 'edit')
     def edit(self, title='', description=''):
         """
@@ -134,17 +163,8 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         return filter( lambda typ, container=self:
                           typ.isConstructionAllowed( container )
                      , result )
-    
-    security.declareProtected(AddPortalFolders, 'manage_addPortalFolder')
-    def manage_addPortalFolder(self, id, title='', REQUEST=None):
-        """Add a new PortalFolder object with id *id*.
-        """
-        ob=PortalFolder(id, title)
-        self._setObject(id, ob)
-        if REQUEST is not None:
-            return self.folder_contents( # XXX: ick!
-                self, REQUEST, portal_status_message="Folder added")
-    
+
+
     def _morphSpec(self, spec):
         '''
         spec is a sequence of meta_types, a string containing one meta type,
@@ -152,6 +172,11 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         meta_types.  Otherwise ensures all of the given meta types are
         contentish.
         '''
+        warn('Using the \'spec\' argument is deprecated. In CMF 2.0 '
+             'contentItems(), contentIds(), contentValues() and '
+             'listFolderContents() will no longer support \'spec\'. Use the '
+             '\'filter\' argument with \'portal_type\' instead.',
+             DeprecationWarning)
         new_spec = []
         types_tool = getToolByName(self, 'portal_types')
         types = types_tool.listContentTypes( by_metatype=1 )
@@ -160,11 +185,10 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
                 spec = [spec]
             for meta_type in spec:
                 if not meta_type in types:
-                    raise ValueError, ('%s is not a content type'
-                                       % meta_type )
+                    raise ValueError('%s is not a content type' % meta_type)
                 new_spec.append(meta_type)
         return new_spec or types
-    
+
     def _filteredItems( self, ids, filt ):
         """
             Apply filter, a mapping, to child objects indicated by 'ids',
@@ -191,91 +215,108 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
             return []
         filt['portal_type'] = pt
 
-        query = apply( ContentFilter, (), filt )
+        query = ContentFilter(**filt)
         result = []
         append = result.append
         get = self._getOb
         for id in ids:
             obj = get( id )
-            include = 0
             if query(obj):
                 append( (id, obj) )
         return result
 
+    #
+    #   'Folderish' interface methods
+    #
+    security.declarePublic('contentItems')
+    def contentItems( self, spec=None, filter=None ):
+        # List contentish and folderish sub-objects and their IDs.
+        # (method is without docstring to disable publishing)
+        #
+        if spec is None:
+            ids = self.objectIds()
+        else:
+            # spec is deprecated, use filter instead!
+            spec = self._morphSpec(spec)
+            ids = self.objectIds(spec)
+        return self._filteredItems( ids, filter )
+
     security.declarePublic('contentIds')
     def contentIds( self, spec=None, filter=None):
-        """
-            Provide a filtered view onto 'objectIds', allowing only
-            PortalFolders and PortalContent-derivatives to show through.
-
-            If 'kw' passed, use them to filter the results further,
-            qua the standard Zope filter interface.
-        """
-        spec = self._morphSpec( spec )
-        ids = self.objectIds( spec )
+        # List IDs of contentish and folderish sub-objects.
+        # (method is without docstring to disable publishing)
+        #
+        if spec is None:
+            ids = self.objectIds()
+        else:
+            # spec is deprecated, use filter instead!
+            spec = self._morphSpec(spec)
+            ids = self.objectIds(spec)
         return map( lambda item: item[0],
                     self._filteredItems( ids, filter ) )
 
     security.declarePublic('contentValues')
     def contentValues( self, spec=None, filter=None ):
-        """
-            Provide a filtered view onto 'objectValues', allowing only
-            PortalFolders and PortalContent-derivatives to show through.
-        """
-        spec = self._morphSpec( spec )
-        ids = self.objectIds( spec )
+        # List contentish and folderish sub-objects.
+        # (method is without docstring to disable publishing)
+        #
+        if spec is None:
+            ids = self.objectIds()
+        else:
+            # spec is deprecated, use filter instead!
+            spec = self._morphSpec(spec)
+            ids = self.objectIds(spec)
         return map( lambda item: item[1],
                     self._filteredItems( ids, filter ) )
 
     security.declareProtected(ListFolderContents, 'listFolderContents')
     def listFolderContents( self, spec=None, contentFilter=None ):
+        """ List viewable contentish and folderish sub-objects.
         """
-            Hook around 'contentValues' to let 'folder_contents'
-            be protected.  Duplicating skip_unauthorized behavior of dtml-in.
-        """
-        items = self.contentValues(spec=spec, filter=contentFilter)
+        items = self.contentItems(spec=spec, filter=contentFilter)
         l = []
-        for obj in items:
-            id = obj.getId()
-            v = obj
+        for id, obj in items:
             # validate() can either raise Unauthorized or return 0 to
             # mean unauthorized.
             try:
-                if getSecurityManager().validate(self, self, id, v):
+                if getSecurityManager().validate(self, self, id, obj):
                     l.append(obj)
-            except Unauthorized:
+            except zExceptions_Unauthorized:  # Catch *all* Unauths!
                 pass
         return l
 
+    #
+    #   webdav Resource method
+    #
 
-    security.declarePublic('contentItems')
-    def contentItems( self, spec=None, filter=None ):
-        """
-            Provide a filtered view onto 'objectItems', allowing only
-            PortalFolders and PortalContent-derivatives to show through.
-        """
-        spec = self._morphSpec( spec )
-        ids = self.objectIds( spec )
-        return self._filteredItems( ids, filter )
+    # protected by 'WebDAV access'
+    def listDAVObjects(self):
+        # List sub-objects for PROPFIND requests.
+        # (method is without docstring to disable publishing)
+        #
+        if _checkPermission(ManagePortal, self):
+            return self.objectValues()
+        else:
+            return self.listFolderContents()
 
+    #
+    #   'DublinCore' interface methods
+    #
     security.declareProtected(View, 'Title')
     def Title( self ):
-        """
-             Implement dublin core Title
+        """ Dublin Core Title element - resource name.
         """
         return self.title
 
     security.declareProtected(View, 'Description')
     def Description( self ):
-        """
-             Implement dublin core Description
+        """ Dublin Core Description element - resource summary.
         """
         return self.description
 
     security.declareProtected(View, 'Type')
     def Type( self ):
-        """
-             Implement dublin core type
+        """ Dublin Core Type element - resource type.
         """
         if hasattr(aq_base(self), 'getTypeInfo'):
             ti = self.getTypeInfo()
@@ -283,6 +324,9 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
                 return ti.Title()
         return self.meta_type
 
+    #
+    #   other methods
+    #
     security.declarePublic('encodeFolderFilter')
     def encodeFolderFilter(self, REQUEST):
         """
@@ -292,8 +336,8 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         for key, value in REQUEST.items():
             if key[:10] == 'filter_by_':
                 filter[key[10:]] = value
-        encoded = string.strip(base64.encodestring( marshal.dumps( filter )))
-        encoded = string.join(string.split(encoded, '\n'), '')
+        encoded = base64.encodestring( marshal.dumps(filter) ).strip()
+        encoded = ''.join( encoded.split('\n') )
         return encoded
 
     security.declarePublic('decodeFolderFilter')
@@ -324,64 +368,53 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
     def reindexObject(self, idxs=[]):
         pass
 
+    def reindexObjectSecurity(self):
+        pass
+
     def PUT_factory( self, name, typ, body ):
+        """ Factory for PUT requests to objects which do not yet exist.
+
+        Used by NullResource.PUT.
+
+        Returns -- Bare and empty object of the appropriate type (or None, if
+        we don't know what to do)
         """
-            Dispatcher for PUT requests to non-existent IDs.  Returns
-            an object of the appropriate type (or None, if we don't
-            know what to do).
-        """
-        registry = getToolByName( self, 'content_type_registry' )
+        registry = getToolByName(self, 'content_type_registry', None)
         if registry is None:
             return None
 
         typeObjectName = registry.findTypeName( name, typ, body )
         if typeObjectName is None:
             return None
-        
+
         self.invokeFactory( typeObjectName, name )
 
-        # XXX: this is butt-ugly.
+        # invokeFactory does too much, so the object has to be removed again
         obj = aq_base( self._getOb( name ) )
         self._delObject( name )
         return obj
 
     security.declareProtected(AddPortalContent, 'invokeFactory')
-    def invokeFactory( self
-                     , type_name
-                     , id
-                     , RESPONSE=None
-                     , *args
-                     , **kw
-                     ):
-        '''
-        Invokes the portal_types tool.
-        '''
-        pt = getToolByName( self, 'portal_types' )
+    def invokeFactory(self, type_name, id, RESPONSE=None, *args, **kw):
+        """ Invokes the portal_types tool.
+        """
+        pt = getToolByName(self, 'portal_types')
         myType = pt.getTypeInfo(self)
 
         if myType is not None:
             if not myType.allowType( type_name ):
-                raise ValueError, 'Disallowed subobject type: %s' % type_name
+                raise ValueError('Disallowed subobject type: %s' % type_name)
 
-        apply( pt.constructContent
-             , (type_name, self, id, RESPONSE) + args
-             , kw
-             )
+        return pt.constructContent(type_name, self, id, RESPONSE, *args, **kw)
 
     security.declareProtected(AddPortalContent, 'checkIdAvailable')
     def checkIdAvailable(self, id):
         try:
             self._checkId(id)
         except BadRequest:
-            return 0
-        except:
-            # Zope < 2.7
-            e, v, tb = sys.exc_info(); del tb
-            if str(e) == 'Bad Request':
-                return 0
-            raise  # Some other exception.
+            return False
         else:
-            return 1
+            return True
 
     def MKCOL_handler(self,id,REQUEST=None,RESPONSE=None):
         """
@@ -390,122 +423,150 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         self.manage_addFolder( id=id, title='' )
 
     def _checkId(self, id, allow_dup=0):
-        PortalFolder.inheritedAttribute('_checkId')(self, id, allow_dup)
-        
-        # This method prevents people other than the portal manager
-        # from overriding skinned names.
-        if not allow_dup:
-            if not _checkPermission( 'Manage portal', self):
-                ob = self
-                while ob is not None and not getattr(ob, '_isPortalRoot', 0):
-                    ob = aq_parent(aq_inner(ob))
-                if ob is not None:
-                    # If the portal root has an object by this name,
-                    # don't allow an override.
-                    # FIXME: needed to allow index_html for join code
-                    if hasattr(ob, id) and id != 'index_html':
-                        raise 'Bad Request', (
-                            'The id "%s" is reserved.' % id)
-                    # Otherwise we're ok.
+        PortalFolderBase.inheritedAttribute('_checkId')(self, id, allow_dup)
+
+        if allow_dup:
+            return
+
+        # FIXME: needed to allow index_html for join code
+        if id == 'index_html':
+            return
+
+        # Another exception: Must allow "syndication_information" to enable
+        # Syndication...
+        if id == 'syndication_information':
+            return
+
+        # This code prevents people other than the portal manager from
+        # overriding skinned names and tools.
+        if not getSecurityManager().checkPermission(ManagePortal, self):
+            ob = self
+            while ob is not None and not getattr(ob, '_isPortalRoot', False):
+                ob = aq_parent( aq_inner(ob) )
+            if ob is not None:
+                # If the portal root has a non-contentish object by this name,
+                # don't allow an override.
+                if (hasattr(ob, id) and
+                    id not in ob.contentIds() and
+                    # Allow root doted prefixed object name overrides
+                    not id.startswith('.')):
+                    raise BadRequest('The id "%s" is reserved.' % id)
+            # Don't allow ids used by Method Aliases.
+            ti = self.getTypeInfo()
+            if ti and ti.queryMethodID(id, context=self):
+                raise BadRequest('The id "%s" is reserved.' % id)
+        # Otherwise we're ok.
 
     def _verifyObjectPaste(self, object, validate_src=1):
         # This assists the version in OFS.CopySupport.
         # It enables the clipboard to function correctly
         # with objects created by a multi-factory.
-        securityChecksDone = 0
-        if (hasattr(object, '__factory_meta_type__') and
-            hasattr(self, 'all_meta_types')):
-            mt = object.__factory_meta_type__
+        securityChecksDone = False
+        sm = getSecurityManager()
+        parent = aq_parent(aq_inner(object))
+        object_id = object.getId()
+        mt = getattr(object, '__factory_meta_type__', None)
+        meta_types = getattr(self, 'all_meta_types', None)
+
+        if mt is not None and meta_types is not None:
             method_name=None
             permission_name = None
-            meta_types = self.all_meta_types
-            if callable(meta_types): meta_types = meta_types()
+
+            if callable(meta_types):
+                meta_types = meta_types()
+
             for d in meta_types:
+
                 if d['name']==mt:
                     method_name=d['action']
                     permission_name = d.get('permission', None)
                     break
 
             if permission_name is not None:
-                if _checkPermission(permission_name,self):
-                    if not validate_src:
-                        # We don't want to check the object on the clipboard
-                        securityChecksDone = 1
-                    else:
-                        try: parent = aq_parent(aq_inner(object))
-                        except: parent = None
-                        if getSecurityManager().validate(None, parent,
-                                                         None, object):
-                            # validation succeeded
-                            securityChecksDone = 1
-                        else:
-                            raise 'Unauthorized', object.getId()
-                else:
-                    raise 'Unauthorized', permission_name
+
+                if not sm.checkPermission(permission_name,self):
+                    raise AccessControl_Unauthorized, method_name
+
+                if validate_src:
+
+                    if not sm.validate(None, parent, None, object):
+                        raise AccessControl_Unauthorized, object_id
+
+                if validate_src > 1:
+                    if not sm.checkPermission(DeleteObjects, parent):
+                        raise AccessControl_Unauthorized
+
+                # validation succeeded
+                securityChecksDone = 1
+
             #
-            # Old validation for objects that may not have registered 
+            # Old validation for objects that may not have registered
             # themselves in the proper fashion.
             #
             elif method_name is not None:
-                meth=self.unrestrictedTraverse(method_name)
-                if hasattr(meth, 'im_self'):
-                    parent = meth.im_self
-                else:
-                    try:    parent = aq_parent(aq_inner(meth))
-                    except: parent = None
-                if getSecurityManager().validate(None, parent, None, meth):
-                    # Ensure the user is allowed to access the object on the
-                    # clipboard.
-                    if not validate_src:
-                        securityChecksDone = 1
-                    else:
-                        try: parent = aq_parent(aq_inner(object))
-                        except: parent = None
-                        if getSecurityManager().validate(None, parent,
-                                                         None, object):
-                            securityChecksDone = 1
-                        else:
-                            raise 'Unauthorized', object.getId()
-                else:
-                    raise 'Unauthorized', method_name
+
+                meth = self.unrestrictedTraverse(method_name)
+
+                factory = getattr(meth, 'im_self', None)
+
+                if factory is None:
+                    factory = aq_parent(aq_inner(meth))
+
+                if not sm.validate(None, factory, None, meth):
+                    raise AccessControl_Unauthorized, method_name
+
+                # Ensure the user is allowed to access the object on the
+                # clipboard.
+                if validate_src:
+
+                    if not sm.validate(None, parent, None, object):
+                        raise AccessControl_Unauthorized, object_id
+
+                if validate_src > 1: # moving
+                    if not sm.checkPermission(DeleteObjects, parent):
+                        raise AccessControl_Unauthorized
+
+                securityChecksDone = 1
 
         # Call OFS' _verifyObjectPaste if necessary
         if not securityChecksDone:
-            PortalFolder.inheritedAttribute(
+            PortalFolderBase.inheritedAttribute(
                 '_verifyObjectPaste')(self, object, validate_src)
 
         # Finally, check allowed content types
-        if hasattr(aq_base(object), '_getPortalTypeName'):
-            contentType = object._getPortalTypeName()
-            if contentType is not None:
+        if hasattr(aq_base(object), 'getPortalTypeName'):
+
+            type_name = object.getPortalTypeName()
+
+            if type_name is not None:
+
                 pt = getToolByName(self, 'portal_types')
                 myType = pt.getTypeInfo(self)
-                if myType is not None and not myType.allowType(contentType):
-                    raise ValueError, \
-                          "Disallowed to paste subobject type '%s'." % contentType
+
+                if myType is not None and not myType.allowType(type_name):
+                    raise ValueError('Disallowed subobject type: %s'
+                                        % type_name)
 
     security.setPermissionDefault(AddPortalContent, ('Owner','Manager'))
-    security.setPermissionDefault(AddPortalFolders, ('Owner','Manager'))
 
+    security.declareProtected(AddPortalFolders, 'manage_addFolder')
     def manage_addFolder( self
                         , id
                         , title=''
                         , REQUEST=None
                         ):
+        """ Add a new folder-like object with id *id*.
+
+        IF present, use the parent object's 'mkdir' alias; otherwise, just add
+        a PortalFolder.
         """
-            Add a new folder-like object with id *id*.  IF present,
-            use the parent object's 'mkdir' action;  otherwise, just
-            add a PortalFolder.
-            to take control of the process by checking for a 'mkdir'
-            action.
-        """
-        try:
-            action = self.getTypeInfo().getActionById( 'mkdir' )
-        except ValueError:
-            self.invokeFactory( type_name='Folder', id=id )
-        else:
+        ti = self.getTypeInfo()
+        method_id = ti and ti.queryMethodID('mkdir', context=self)
+        if method_id:
             # call it
-            getattr( self, action )( id=id )
+            getattr(self, method_id)(id=id)
+        else:
+            self.invokeFactory( type_name='Folder', id=id )
 
         ob = self._getOb( id )
         ob.setTitle( title )
@@ -517,8 +578,35 @@ class PortalFolder(DynamicType, CMFCatalogAware, Folder):
         if REQUEST is not None:
             return self.manage_main(self, REQUEST, update_menu=1)
 
-Globals.InitializeClass(PortalFolder)
-    
+InitializeClass(PortalFolderBase)
+
+
+class PortalFolder(OrderSupport, PortalFolderBase):
+    """
+        Implements portal content management, but not UI details.
+    """
+    meta_type = 'Portal Folder'
+    portal_type = 'Folder'
+
+    __implements__ = (PortalFolderBase.__implements__,
+                      OrderSupport.__implements__)
+
+    security = ClassSecurityInfo()
+
+    manage_options = ( OrderSupport.manage_options +
+                       PortalFolderBase.manage_options[1:] )
+
+    security.declareProtected(AddPortalFolders, 'manage_addPortalFolder')
+    def manage_addPortalFolder(self, id, title='', REQUEST=None):
+        """Add a new PortalFolder object with id *id*.
+        """
+        ob = PortalFolder(id, title)
+        self._setObject(id, ob)
+        if REQUEST is not None:
+            return self.folder_contents( # XXX: ick!
+                self, REQUEST, portal_status_message="Folder added")
+
+InitializeClass(PortalFolder)
 
 
 class ContentFilter:
@@ -544,28 +632,27 @@ class ContentFilter:
         self.predicates = []
         self.description = []
 
-        if Title is not self.MARKER: 
+        if Title is not self.MARKER:
             self.predicates.append( lambda x, pat=re.compile( Title ):
                                       pat.search( x.Title() ) )
             self.description.append( 'Title: %s' % Title )
 
-        if Creator is not self.MARKER: 
-            self.predicates.append( lambda x, pat=re.compile( Creator ):
-                                      pat.search( x.Creator() ) )
+        if Creator and Creator is not self.MARKER:
+            self.predicates.append( lambda x, creator=Creator:
+                                    creator in x.listCreators() )
             self.description.append( 'Creator: %s' % Creator )
 
-        if Subject and Subject is not self.MARKER: 
+        if Subject and Subject is not self.MARKER:
             self.filterSubject = Subject
             self.predicates.append( self.hasSubject )
-            self.description.append( 'Subject: %s'
-                                   % string.join( Subject, ', ' ) )
+            self.description.append( 'Subject: %s' % ', '.join(Subject) )
 
-        if Description is not self.MARKER: 
+        if Description is not self.MARKER:
             self.predicates.append( lambda x, pat=re.compile( Description ):
                                       pat.search( x.Description() ) )
             self.description.append( 'Description: %s' % Description )
 
-        if created is not self.MARKER: 
+        if created is not self.MARKER:
             if created_usage == 'range:min':
                 self.predicates.append( lambda x, cd=created:
                                           cd <= x.created() )
@@ -575,7 +662,7 @@ class ContentFilter:
                                           cd >= x.created() )
                 self.description.append( 'Created before: %s' % created )
 
-        if modified is not self.MARKER: 
+        if modified is not self.MARKER:
             if modified_usage == 'range:min':
                 self.predicates.append( lambda x, md=modified:
                                           md <= x.modified() )
@@ -590,16 +677,16 @@ class ContentFilter:
                 Type = [ Type ]
             self.predicates.append( lambda x, Type=Type:
                                       x.Type() in Type )
-            self.description.append( 'Type: %s'
-                                   % string.join( Type, ', ' ) )
+            self.description.append( 'Type: %s' % ', '.join(Type) )
 
         if portal_type and portal_type is not self.MARKER:
             if type(portal_type) is type(''):
                 portal_type = [portal_type]
-            self.predicates.append(lambda x, pt=portal_type:
-                                   x.getPortalTypeName() in pt)
-            self.description.append('Portal Type: %s'
-                                    % string.join(portal_type, ', '))
+            self.predicates.append( lambda x, pt=portal_type:
+                                    hasattr(aq_base(x), 'getPortalTypeName')
+                                    and x.getPortalTypeName() in pt )
+            self.description.append( 'Portal Type: %s'
+                                     % ', '.join(portal_type) )
 
     def hasSubject( self, obj ):
         """
@@ -627,7 +714,7 @@ class ContentFilter:
         """
             Return a stringified description of the filter.
         """
-        return string.join( self.description, '; ' )
+        return '; '.join(self.description)
 
-manage_addPortalFolder = PortalFolder.manage_addPortalFolder
+manage_addPortalFolder = PortalFolder.manage_addPortalFolder.im_func
 manage_addPortalFolderForm = DTMLFile( 'folderAdd', globals() )
