@@ -1,40 +1,40 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
-# 
+# FOR A PARTICULAR PURPOSE.
+#
 ##############################################################################
-
 """ Basic workflow tool.
 
-$Id: WorkflowTool.py,v 1.24.4.10 2002/08/04 23:33:30 efge Exp $
+$Id: WorkflowTool.py 40138 2005-11-15 17:47:37Z jens $
 """
 
 import sys
-from string import join, split, replace, strip
+from warnings import warn
 
-from OFS.Folder import Folder
-from Globals import InitializeClass, PersistentMapping, DTMLFile
 from AccessControl import ClassSecurityInfo
 from Acquisition import aq_base, aq_inner, aq_parent
+from Globals import DTMLFile
+from Globals import InitializeClass
+from Globals import PersistentMapping
+from OFS.Folder import Folder
 
-from CMFCorePermissions import ManagePortal
-from WorkflowCore import WorkflowException, ObjectDeleted, ObjectMoved
-
-from interfaces.portal_workflow import portal_workflow
-
-from utils import UniqueObject
-from utils import _checkPermission
-from utils import getToolByName
+from ActionProviderBase import ActionProviderBase
+from interfaces.portal_workflow import portal_workflow as IWorkflowTool
+from permissions import ManagePortal
 from utils import _dtmldir
+from utils import getToolByName
+from utils import UniqueObject
+from WorkflowCore import ObjectDeleted
+from WorkflowCore import ObjectMoved
+from WorkflowCore import WorkflowException
 
-AUTO_MIGRATE_WORKFLOW_TOOLS = 0  # Set to 1 to auto-migrate
 
 _marker = []  # Create a new marker object.
 
@@ -44,7 +44,10 @@ class WorkflowInformation:
         querying actions without mediation of the 'portal_actions' tool.
     """
     def __init__(self, object):
-        self.content = object
+        warn('WorkflowInformation() is deprecated and will be removed in '
+             'CMF 2.0.',
+             DeprecationWarning)
+        self.object = self.content = object
         self.content_url = object.absolute_url()
         self.portal_url = self.folder_url = ''
 
@@ -56,13 +59,13 @@ class WorkflowInformation:
         raise KeyError, name
 
 
-class WorkflowTool (UniqueObject, Folder):
-
-    """ Mediator tool, mapping workflow objects 
+class WorkflowTool(UniqueObject, Folder, ActionProviderBase):
+    """ Mediator tool, mapping workflow objects
     """
     id = 'portal_workflow'
     meta_type = 'CMF Workflow Tool'
-    __implements__ = portal_workflow
+    __implements__ = (IWorkflowTool,
+                      ActionProviderBase.__implements__)
 
     _chains_by_type = None  # PersistentMapping
     _default_chain = ('default_workflow',)
@@ -82,24 +85,6 @@ class WorkflowTool (UniqueObject, Folder):
     security.declareProtected( ManagePortal, 'manage_overview' )
     manage_overview = DTMLFile( 'explainWorkflowTool', _dtmldir )
 
-    if AUTO_MIGRATE_WORKFLOW_TOOLS:
-        def __setstate__(self, state):
-            # Adds default_workflow to persistent WorkflowTool instances.
-            # This is temporary!
-            WorkflowTool.inheritedAttribute('__setstate__')(self, state)
-            if not self.__dict__.has_key('default_workflow'):
-                try:
-                    from Products.CMFDefault import DefaultWorkflow
-                except ImportError:
-                    pass
-                else:
-                    self.default_workflow = (
-                        DefaultWorkflow.DefaultWorkflowDefinition(
-                        'default_workflow'))
-                    self._objects = self._objects + (
-                        {'id': 'default_workflow',
-                         'meta_type': self.default_workflow.meta_type},)
-
     _manage_addWorkflowForm = DTMLFile('addWorkflow', _dtmldir)
 
     security.declareProtected( ManagePortal, 'manage_addWorkflowForm')
@@ -147,7 +132,7 @@ class WorkflowTool (UniqueObject, Folder):
             if title == id:
                 title = None
             if cbt is not None and cbt.has_key(id):
-                chain = join(cbt[id], ', ')
+                chain = ', '.join(cbt[id])
             else:
                 chain = '(Default)'
             types_info.append({'id': id,
@@ -155,7 +140,7 @@ class WorkflowTool (UniqueObject, Folder):
                                'chain': chain})
         return self._manage_selectWorkflows(
             REQUEST,
-            default_chain=join(self._default_chain, ', '),
+            default_chain=', '.join(self._default_chain),
             types_info=types_info,
             management_view='Workflows',
             manage_tabs_message=manage_tabs_message)
@@ -172,28 +157,29 @@ class WorkflowTool (UniqueObject, Folder):
             self._chains_by_type = cbt = PersistentMapping()
         ti = self._listTypeInfo()
         # Set up the chains by type.
-        for t in ti:
-            id = t.getId()
-            field_name = 'chain_%s' % id
-            chain = strip(props.get(field_name, '(Default)'))
-            if chain == '(Default)':
-                # Remove from cbt.
-                if cbt.has_key(id):
-                    del cbt[id]
-            else:
-                chain = replace(chain, ',', ' ')
-                ids = []
-                for wf_id in split(chain, ' '):
-                    if wf_id:
-                        if not self.getWorkflowById(wf_id):
-                            raise ValueError, (
-                                '"%s" is not a workflow ID.' % wf_id)
-                        ids.append(wf_id)
-                cbt[id] = tuple(ids)
+        if not (props is None):
+            for t in ti:
+                id = t.getId()
+                field_name = 'chain_%s' % id
+                chain = props.get(field_name, '(Default)').strip()
+                if chain == '(Default)':
+                    # Remove from cbt.
+                    if cbt.has_key(id):
+                        del cbt[id]
+                else:
+                    chain = chain.replace(',', ' ')
+                    ids = []
+                    for wf_id in chain.split(' '):
+                        if wf_id:
+                            if not self.getWorkflowById(wf_id):
+                                raise ValueError, (
+                                    '"%s" is not a workflow ID.' % wf_id)
+                            ids.append(wf_id)
+                    cbt[id] = tuple(ids)
         # Set up the default chain.
-        default_chain = replace(default_chain, ',', ' ')
+        default_chain = default_chain.replace(',', ' ')
         ids = []
-        for wf_id in split(default_chain, ' '):
+        for wf_id in default_chain.split(' '):
             if wf_id:
                 if not self.getWorkflowById(wf_id):
                     raise ValueError, (
@@ -213,7 +199,7 @@ class WorkflowTool (UniqueObject, Folder):
         """ Returns a mapping of the catalog variables that apply to ob.
 
         o Invoked by portal_catalog.
-        
+
         o Allows workflows to add variables to the catalog based on
           workflow status, making it possible to implement queues.
         """
@@ -231,22 +217,25 @@ class WorkflowTool (UniqueObject, Folder):
         return vars
 
     security.declarePrivate('listActions')
-    def listActions(self, info):
+    def listActions(self, info=None, object=None):
 
         """ Returns a list of actions to be displayed to the user.
 
         o Invoked by the portal_actions tool.
-        
+
         o Allows workflows to include actions to be displayed in the
           actions box.
 
         o Object actions are supplied by workflows that apply to the object.
-        
+
         o Global actions are supplied by all workflows.
         """
-        chain = self.getChainFor(info.content)
+        if object is not None or info is None:
+            info = self._getOAI(object)
+        chain = self.getChainFor(info.object)
         did = {}
         actions = []
+
         for wf_id in chain:
             did[wf_id] = 1
             wf = self.getWorkflowById(wf_id)
@@ -274,17 +263,20 @@ class WorkflowTool (UniqueObject, Folder):
         """ Return a list of action dictionaries for 'ob', just as though
             queried via 'ActionsTool.listFilteredActionsFor'.
         """
+        warn('getActionsFor() is deprecated and will be removed in CMF 2.0. '
+             'Please use listActionInfos() instead.',
+             DeprecationWarning)
         return self.listActions( WorkflowInformation( ob ) )
 
     security.declarePublic('doActionFor')
     def doActionFor(self, ob, action, wf_id=None, *args, **kw):
 
         """ Execute the given workflow action for the object.
-        
+
         o Invoked by user interface code.
 
         o Allows the user to request a workflow action.
-        
+
         o The workflow object must perform its own security checks.
         """
         wfs = self.getWorkflowsFor(ob)
@@ -295,7 +287,7 @@ class WorkflowTool (UniqueObject, Folder):
                 raise WorkflowException('No workflows found.')
             found = 0
             for wf in wfs:
-                if wf.isActionSupported(ob, action):
+                if wf.isActionSupported(ob, action, **kw):
                     found = 1
                     break
             if not found:
@@ -313,11 +305,11 @@ class WorkflowTool (UniqueObject, Folder):
     def getInfoFor(self, ob, name, default=_marker, wf_id=None, *args, **kw):
 
         """ Return a given workflow-specific property for an object.
-        
+
         o Invoked by user interface code.
-        
+
         o Allows the user to request information provided by the workflow.
-        
+
         o The workflow object must perform its own security checks.
         """
         if wf_id is None:
@@ -346,7 +338,7 @@ class WorkflowTool (UniqueObject, Folder):
                         'Requested workflow definition not found.')
                 else:
                     return default
-        res = apply(wf.getInfoFor, (ob, name, default) + args, kw)
+        res = wf.getInfoFor(ob, name, default, *args, **kw)
         if res is _marker:
             raise WorkflowException('Could not get info: %s' % name)
         return res
@@ -367,7 +359,7 @@ class WorkflowTool (UniqueObject, Folder):
 
         """ Notifies all applicable workflows of an action before it
             happens, allowing veto by exception.
-            
+
         o Unless an exception is thrown, either a notifySuccess() or
           notifyException() can be expected later on.
 
@@ -399,7 +391,7 @@ class WorkflowTool (UniqueObject, Folder):
     def getHistoryOf(self, wf_id, ob):
 
         """ Return the history of an object.
-        
+
         o Invoked by workflow definitions.
         """
         if hasattr(aq_base(ob), 'workflow_history'):
@@ -450,9 +442,9 @@ class WorkflowTool (UniqueObject, Folder):
 
         """ Set the default chain for this tool
         """
-        default_chain = replace(default_chain, ',', ' ')
+        default_chain = default_chain.replace(',', ' ')
         ids = []
-        for wf_id in split(default_chain, ' '):
+        for wf_id in default_chain.split(' '):
             if wf_id:
                 if not self.getWorkflowById(wf_id):
                     raise ValueError, ( '"%s" is not a workflow ID.' % wf_id)
@@ -469,8 +461,8 @@ class WorkflowTool (UniqueObject, Folder):
         if cbt is None:
             self._chains_by_type = cbt = PersistentMapping()
 
-        if type(chain) is type(''):
-            chain = map(strip, split(chain,','))
+        if isinstance(chain, basestring):
+            chain = [ wf.strip() for wf in chain.split(',') if wf.strip() ]
 
         ti = self._listTypeInfo()
         for t in ti:
@@ -528,10 +520,10 @@ class WorkflowTool (UniqueObject, Folder):
             the portal_type.
         """
         cbt = self._chains_by_type
-        if type(ob) == type(''):
+        if isinstance(ob, basestring):
             pt = ob
-        elif hasattr(aq_base(ob), '_getPortalTypeName'):
-            pt = ob._getPortalTypeName()
+        elif hasattr(aq_base(ob), 'getPortalTypeName'):
+            pt = ob.getPortalTypeName()
         else:
             pt = None
 
@@ -562,7 +554,7 @@ class WorkflowTool (UniqueObject, Folder):
 
         return tuple(wf_ids)
 
-    security.declarePrivate('getWorkflowsFor')
+    security.declareProtected(ManagePortal, 'getWorkflowsFor')
     def getWorkflowsFor(self, ob):
 
         """ Find the workflows for the type of the given object.
@@ -592,7 +584,7 @@ class WorkflowTool (UniqueObject, Folder):
             wfs = ()
         if wf is None:
             # No workflow wraps this method.
-            return apply(func, args, kw)
+            return func(*args, **kw)
         return self._invokeWithNotification(
             wfs, ob, method_id, wf.wrapWorkflowMethod,
             (ob, method_id, func, args, kw), {})
@@ -621,7 +613,7 @@ class WorkflowTool (UniqueObject, Folder):
         for w in wfs:
             w.notifyBefore(ob, action)
         try:
-            res = apply(func, args, kw)
+            res = func(*args, **kw)
         except ObjectDeleted, ex:
             res = ex.getResult()
             reindex = 0
@@ -686,12 +678,14 @@ class WorkflowTool (UniqueObject, Folder):
             the catalog (sometimes not desirable, e.g. when the workflow
             objects do this themselves only at particular points).
         """
-        self._default_cataloging = not not value
+        self._default_cataloging = bool(value)
 
     security.declarePrivate('_reindexWorkflowVariables')
     def _reindexWorkflowVariables(self, ob):
 
         """ Reindex the variables that the workflow may have changed.
+
+        Also reindexes the security.
         """
         if not self._default_cataloging:
             return
@@ -699,10 +693,13 @@ class WorkflowTool (UniqueObject, Folder):
         if hasattr(aq_base(ob), 'reindexObject'):
             # XXX We only need the keys here, no need to compute values.
             mapping = self.getCatalogVariablesFor(ob) or {}
-            mapping['allowedRolesAndUsers'] = None
             vars = mapping.keys()
             ob.reindexObject(idxs=vars)
 
+        # Reindex security of subobjects.
+        if hasattr(aq_base(ob), 'reindexObjectSecurity'):
+            ob.reindexObjectSecurity()
+
 InitializeClass(WorkflowTool)
 
 
