@@ -1,24 +1,25 @@
 ##############################################################################
 #
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
-# 
+#
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
-# 
+# FOR A PARTICULAR PURPOSE.
+#
 ##############################################################################
+""" Caching policies tool interface.
 
-"""Caching tool interface description.
-
-$Id: CachingPolicyManager.py,v 1.3 2002/07/03 18:01:35 tseaver Exp $
+$Id: CachingPolicyManager.py 36457 2004-08-12 15:07:44Z jens $
 """
 
-from Interface import Attribute, Base
+from Interface import Attribute
+from Interface import Interface
 
-class CachingPolicyManager( Base ):
+
+class CachingPolicyManager(Interface):
     """
         Manage HTTP cache policies for skin methods.
     """
