@@ -3,33 +3,44 @@
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+""" Dublin Core support for content types.
 
-import string, re
-from OFS.PropertyManager import PropertyManager
-from DateTime.DateTime import DateTime
-from Acquisition import aq_base
-from Products.CMFCore.WorkflowCore import WorkflowAction
-from Products.CMFCore.interfaces.DublinCore import DublinCore
-from Products.CMFCore.interfaces.DublinCore import CatalogableDublinCore
-from Products.CMFCore.interfaces.DublinCore import MutableDublinCore
+$Id: DublinCore.py 36913 2005-04-09 23:00:10Z slinkp $
+"""
 
-from utils import tuplize, _dtmldir, semi_split
-from Globals import InitializeClass, DTMLFile
 from AccessControl import ClassSecurityInfo
-from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
+from Acquisition import aq_base
+from DateTime.DateTime import DateTime
+from Globals import DTMLFile
+from Globals import InitializeClass
+from OFS.PropertyManager import PropertyManager
+
+from Products.CMFCore.interfaces.DublinCore import CatalogableDublinCore
+from Products.CMFCore.interfaces.DublinCore import DublinCore
+from Products.CMFCore.interfaces.DublinCore import MutableDublinCore
+from Products.CMFCore.utils import getToolByName
+
+from permissions import ModifyPortalContent
+from permissions import View
+from utils import tuplize
+from utils import _dtmldir
+from utils import semi_split
 
 _marker=[]
 
+# For http://www.zope.org/Collectors/CMF/325
+# We only really need this once, at startup.
+_zone = DateTime().timezone()
+    
 class DefaultDublinCoreImpl( PropertyManager ):
-    """
-        Mix-in class which provides Dublin Core methods
+    """ Mix-in class which provides Dublin Core methods.
     """
     __implements__ = DublinCore, CatalogableDublinCore, MutableDublinCore
 
@@ -49,6 +60,7 @@ class DefaultDublinCoreImpl( PropertyManager ):
         now = DateTime()
         self.creation_date = now
         self.modification_date = now
+        self.creators = ()
         self._editMetadata( title
                           , subject
                           , description
@@ -70,19 +82,30 @@ class DefaultDublinCoreImpl( PropertyManager ):
 
     security.declarePrivate('notifyModified')
     def notifyModified(self):
+        """ Take appropriate action after the resource has been modified.
+
+        Update creators and modification_date.
         """
-        Take appropriate action after the resource has been modified.
-        For now, change the modification_date.
-        """
-        # XXX This could also store the id of the user doing modifications.
+        self.addCreator()
         self.setModificationDate()
 
-    # XXX Could this be simply protected by ModifyPortalContent ?
-    security.declarePrivate('setModificationDate')
-    def setModificationDate(self, modification_date=None):
+    security.declareProtected(ModifyPortalContent, 'addCreator')
+    def addCreator(self, creator=None):
+        """ Add creator to Dublin Core creators.
         """
-            Set the date when the resource was last modified.
-            When called without an argument, sets the date to now.
+        if creator is None:
+            mtool = getToolByName(self, 'portal_membership', None)
+            creator = mtool and mtool.getAuthenticatedMember().getId()
+
+        # call self.listCreators() to make sure self.creators exists
+        if creator and not creator in self.listCreators():
+            self.creators = self.creators + (creator, )
+
+    security.declareProtected(ModifyPortalContent, 'setModificationDate')
+    def setModificationDate(self, modification_date=None):
+        """ Set the date when the resource was last modified.
+
+        When called without an argument, sets the date to now.
         """
         if modification_date is None:
             self.modification_date = DateTime()
@@ -92,117 +115,140 @@ class DefaultDublinCoreImpl( PropertyManager ):
     #
     #  DublinCore interface query methods
     #
-    security.declarePublic( 'Title' )
+    security.declareProtected(View, 'Title')
     def Title( self ):
-        "Dublin Core element - resource name"
+        """ Dublin Core Title element - resource name.
+        """
         return self.title
 
-    security.declarePublic( 'Creator' )
-    def Creator( self ):
-        # XXX: fixme using 'portal_membership' -- should iterate over
-        #       *all* owners
-        "Dublin Core element - resource creator"
-        owner = self.getOwner()
-        if hasattr( owner, 'getId' ):
-            return owner.getId()
-        return 'No owner'
+    security.declareProtected(View, 'listCreators')
+    def listCreators(self):
+        """ List Dublin Core Creator elements - resource authors.
+        """
+        if not hasattr(aq_base(self), 'creators'):
+            # for content created with CMF versions before 1.5
+            owner_tuple = self.getOwnerTuple()
+            if owner_tuple:
+                self.creators = (owner_tuple[1],)
+            else:
+                self.creators = ()
+        return self.creators
 
-    security.declarePublic( 'Subject' )
+    security.declareProtected(View, 'Creator')
+    def Creator(self):
+        """ Dublin Core Creator element - resource author.
+        """
+        creators = self.listCreators()
+        return creators and creators[0] or ''
+
+    security.declareProtected(View, 'Subject')
     def Subject( self ):
-        "Dublin Core element - resource keywords"
+        """ Dublin Core Subject element - resource keywords.
+        """
         return getattr( self, 'subject', () ) # compensate for *old* content
 
-    security.declarePublic( 'Publisher' )
-    def Publisher( self ):
-        "Dublin Core element - resource publisher"
-        # XXX: fixme using 'portal_metadata'
-        return 'No publisher'
-
-    security.declarePublic( 'Description' )
+    security.declareProtected(View, 'Description')
     def Description( self ):
-        "Dublin Core element - resource summary"
+        """ Dublin Core Description element - resource summary.
+        """
         return self.description
 
-    security.declarePublic( 'Contributors' )
-    def Contributors( self ):
-        "Dublin Core element - additional contributors to resource"
-        # XXX: fixme
+    security.declareProtected(View, 'Publisher')
+    def Publisher( self ):
+        """ Dublin Core Publisher element - resource publisher.
+        """
+        tool = getToolByName(self, 'portal_metadata', None)
+
+        if tool is not None:
+            return tool.getPublisher()
+
+        return 'No publisher'
+
+    security.declareProtected(View, 'listContributors')
+    def listContributors(self):
+        """ Dublin Core Contributor elements - resource collaborators.
+        """
         return self.contributors
 
-    security.declarePublic( 'Date' )
+    security.declareProtected(View, 'Contributors')
+    def Contributors(self):
+        """ Deprecated alias of listContributors.
+        """
+        return self.listContributors()
+
+    security.declareProtected(View, 'Date')
     def Date( self ):
-        "Dublin Core element - default date"
+        """ Dublin Core Date element - default date.
+        """
         # Return effective_date if set, modification date otherwise
         date = getattr(self, 'effective_date', None )
         if date is None:
             date = self.modified()
-        return date.ISO()
+        return date.toZone(_zone).ISO()
 
-    security.declarePublic( 'CreationDate' )
+    security.declareProtected(View, 'CreationDate')
     def CreationDate( self ):
-        """
-            Dublin Core element - date resource created.
+        """ Dublin Core Date element - date resource created.
         """
         # return unknown if never set properly
-        return self.creation_date and self.creation_date.ISO() or 'Unknown'
+        if self.creation_date:
+            return self.creation_date.toZone(_zone).ISO()
+        else:
+            return 'Unknown'
 
-    security.declarePublic( 'EffectiveDate' )
+    security.declareProtected(View, 'EffectiveDate')
     def EffectiveDate( self ):
-        """
-            Dublin Core element - date resource becomes effective.
+        """ Dublin Core Date element - date resource becomes effective.
         """
         ed = getattr( self, 'effective_date', None )
-        return ed and ed.ISO() or 'None'
+        return ed and ed.toZone(_zone).ISO() or 'None'
 
-    security.declarePublic( 'ExpirationDate' )
+    security.declareProtected(View, 'ExpirationDate')
     def ExpirationDate( self ):
-        """
-            Dublin Core element - date resource expires.
+        """ Dublin Core Date element - date resource expires.
         """
         ed = getattr( self, 'expiration_date', None )
-        return ed and ed.ISO() or 'None'
+        return ed and ed.toZone(_zone).ISO() or 'None'
 
-    security.declarePublic( 'ModificationDate' )
+    security.declareProtected(View, 'ModificationDate')
     def ModificationDate( self ):
+        """ Dublin Core Date element - date resource last modified.
         """
-            Dublin Core element - date resource last modified.
-        """
-        return self.modified().ISO()
+        return self.modified().toZone(_zone).ISO()
 
-    security.declarePublic( 'Type' )
+    security.declareProtected(View, 'Type')
     def Type( self ):
-        "Dublin Core element - Object type"
+        """ Dublin Core Type element - resource type.
+        """
         if hasattr(aq_base(self), 'getTypeInfo'):
             ti = self.getTypeInfo()
             if ti is not None:
                 return ti.Title()
         return self.meta_type
 
-    security.declarePublic( 'Format' )
+    security.declareProtected(View, 'Format')
     def Format( self ):
-        """
-            Dublin Core element - resource format
+        """ Dublin Core Format element - resource format.
         """
         return self.format
 
-    security.declarePublic( 'Identifier' )
+    security.declareProtected(View, 'Identifier')
     def Identifier( self ):
-        "Dublin Core element - Object ID"
+        """ Dublin Core Identifier element - resource ID.
+        """
         # XXX: fixme using 'portal_metadata' (we need to prepend the
         #      right prefix to self.getPhysicalPath().
         return self.absolute_url()
 
-    security.declarePublic( 'Language' )
+    security.declareProtected(View, 'Language')
     def Language( self ):
-        """
-            Dublin Core element - resource language
+        """ Dublin Core Language element - resource language.
         """
         return self.language
 
-    security.declarePublic( 'Rights' )
+    security.declareProtected(View, 'Rights')
     def Rights( self ):
-        """
-            Dublin Core element - resource copyright
+        """ Dublin Core Rights element - resource copyright.
         """
         return self.rights
 
@@ -210,17 +256,15 @@ class DefaultDublinCoreImpl( PropertyManager ):
     #  DublinCore utility methods
     #
     def content_type( self ):
-        """
-            WebDAV needs this to do the Right Thing (TM).
+        """ WebDAV needs this to do the Right Thing (TM).
         """
         return self.Format()
 
     __FLOOR_DATE = DateTime( 1970, 0 ) # always effective
 
-    security.declarePublic( 'isEffective' )
+    security.declareProtected(View, 'isEffective')
     def isEffective( self, date ):
-        """
-            Is the date within the resource's effective range?
+        """ Is the date within the resource's effective range?
         """
         pastEffective = ( self.effective_date is None
                        or self.effective_date <= date )
@@ -231,21 +275,17 @@ class DefaultDublinCoreImpl( PropertyManager ):
     #
     #  CatalogableDublinCore methods
     #
-    security.declarePublic( 'created' )
+    security.declareProtected(View, 'created')
     def created( self ):
-        """
-            Dublin Core element - date resource created,
-              returned as DateTime.
+        """ Dublin Core Date element - date resource created.
         """
         # allow for non-existent creation_date, existed always
         date = getattr( self, 'creation_date', None )
         return date is None and self.__FLOOR_DATE or date
 
-    security.declarePublic( 'effective' )
+    security.declareProtected(View, 'effective')
     def effective( self ):
-        """
-            Dublin Core element - date resource becomes effective,
-              returned as DateTime.
+        """ Dublin Core Date element - date resource becomes effective.
         """
         marker = []
         date = getattr( self, 'effective_date', marker )
@@ -253,22 +293,18 @@ class DefaultDublinCoreImpl( PropertyManager ):
             date = getattr( self, 'creation_date', None )
         return date is None and self.__FLOOR_DATE or date
 
-    __CEILING_DATE = DateTime( 9999, 0 ) # never expires
+    __CEILING_DATE = DateTime( 2500, 0 ) # never expires
 
-    security.declarePublic( 'expires' )
+    security.declareProtected(View, 'expires')
     def expires( self ):
-        """
-            Dublin Core element - date resource expires,
-              returned as DateTime.
+        """ Dublin Core Date element - date resource expires.
         """
         date = getattr( self, 'expiration_date', None )
         return date is None and self.__CEILING_DATE or date
 
-    security.declarePublic( 'modified' )
+    security.declareProtected(View, 'modified')
     def modified( self ):
-        """
-            Dublin Core element - date resource last modified,
-              returned as DateTime.
+        """ Dublin Core Date element - date resource last modified.
         """
         date = self.modification_date
         if date is None:
@@ -277,18 +313,16 @@ class DefaultDublinCoreImpl( PropertyManager ):
             self.modification_date = date
         return date
 
-    security.declarePublic( 'getMetadataHeaders' )
+    security.declareProtected(View, 'getMetadataHeaders')
     def getMetadataHeaders( self ):
-        """
-            Return RFC-822-style headers.
+        """ Return RFC-822-style headers.
         """
         hdrlist = []
         hdrlist.append( ( 'Title', self.Title() ) )
-        hdrlist.append( ( 'Subject', string.join( self.Subject(), ', ' ) ) )
+        hdrlist.append( ( 'Subject', ', '.join( self.Subject() ) ) )
         hdrlist.append( ( 'Publisher', self.Publisher() ) )
         hdrlist.append( ( 'Description', self.Description() ) )
-        hdrlist.append( ( 'Contributors', string.join(
-            self.Contributors(), '; ' ) ) )
+        hdrlist.append( ( 'Contributors', '; '.join( self.Contributors() ) ) )
         hdrlist.append( ( 'Effective_date', self.EffectiveDate() ) )
         hdrlist.append( ( 'Expiration_date', self.ExpirationDate() ) )
         hdrlist.append( ( 'Type', self.Type() ) )
@@ -311,57 +345,62 @@ class DefaultDublinCoreImpl( PropertyManager ):
 
     security.declareProtected(ModifyPortalContent, 'setTitle')
     def setTitle( self, title ):
-        "Dublin Core element - resource name"
+        """ Set Dublin Core Title element - resource name.
+        """
         self.title = title
 
+    security.declareProtected(ModifyPortalContent, 'setCreators')
+    def setCreators(self, creators):
+        """ Set Dublin Core Creator elements - resource authors.
+        """
+        self.creators = tuplize('creators', creators)
+
     security.declareProtected(ModifyPortalContent, 'setSubject')
     def setSubject( self, subject ):
-        "Dublin Core element - resource keywords"
+        """ Set Dublin Core Subject element - resource keywords.
+        """
         self.subject = tuplize( 'subject', subject )
 
     security.declareProtected(ModifyPortalContent, 'setDescription')
     def setDescription( self, description ):
-        "Dublin Core element - resource summary"
+        """ Set Dublin Core Description element - resource summary.
+        """
         self.description = description
 
     security.declareProtected(ModifyPortalContent, 'setContributors')
     def setContributors( self, contributors ):
-        "Dublin Core element - additional contributors to resource"
+        """ Set Dublin Core Contributor elements - resource collaborators.
+        """
         # XXX: fixme
         self.contributors = tuplize('contributors', contributors, semi_split)
 
     security.declareProtected(ModifyPortalContent, 'setEffectiveDate')
     def setEffectiveDate( self, effective_date ):
-        """
-            Dublin Core element - date resource becomes effective.
+        """ Set Dublin Core Date element - date resource becomes effective.
         """
         self.effective_date = self._datify( effective_date )
 
     security.declareProtected(ModifyPortalContent, 'setExpirationDate')
     def setExpirationDate( self, expiration_date ):
-        """
-            Dublin Core element - date resource expires.
+        """ Set Dublin Core Date element - date resource expires.
         """
         self.expiration_date = self._datify( expiration_date )
 
     security.declareProtected(ModifyPortalContent, 'setFormat')
     def setFormat( self, format ):
-        """
-            Dublin Core element - resource format
+        """ Set Dublin Core Format element - resource format.
         """
         self.format = format
 
     security.declareProtected(ModifyPortalContent, 'setLanguage')
     def setLanguage( self, language ):
-        """
-            Dublin Core element - resource language
+        """ Set Dublin Core Language element - resource language.
         """
         self.language = language
 
     security.declareProtected(ModifyPortalContent, 'setRights')
     def setRights( self, rights ):
-        """
-            Dublin Core element - resource copyright
+        """ Set Dublin Core Rights element - resource copyright.
         """
         self.rights = rights
 
@@ -381,8 +420,7 @@ class DefaultDublinCoreImpl( PropertyManager ):
                      , language=_marker
                      , rights=_marker
                      ):
-        """
-            Update the editable metadata for this resource.
+        """ Update the editable metadata for this resource.
         """
         if title is not _marker:
             self.setTitle( title )
@@ -419,8 +457,7 @@ class DefaultDublinCoreImpl( PropertyManager ):
                            , rights
                            , REQUEST
                            ):
-        """
-            Update metadata from the ZMI.
+        """ Update metadata from the ZMI.
         """
         self._editMetadata( title, subject, description, contributors
                           , effective_date, expiration_date
@@ -443,7 +480,6 @@ class DefaultDublinCoreImpl( PropertyManager ):
                    , rights=''
                     ):
         """
-        used to be:  editMetadata = WorkflowAction(_editMetadata)
         Need to add check for webDAV locked resource for TTW methods.
         """
         # as per bug #69, we cant assume they use the webdav
