@@ -3,32 +3,35 @@
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """ CMFDefault portal_metadata tool.
 
-$Id: MetadataTool.py,v 1.20.20.1 2004/04/22 17:47:50 sidnei Exp $
+$Id: MetadataTool.py 37005 2005-05-04 21:33:28Z jens $
 """
 
-from OFS.SimpleItem import SimpleItem
-from Products.CMFCore.utils import UniqueObject
+from AccessControl import ClassSecurityInfo
+from AccessControl import getSecurityManager
+from Globals import DTMLFile
+from Globals import InitializeClass
 from Globals import PersistentMapping
+from OFS.SimpleItem import SimpleItem
 
-from Globals import InitializeClass, DTMLFile
-from AccessControl import ClassSecurityInfo, getSecurityManager
-from Products.CMFCore.CMFCorePermissions import View
-from Products.CMFCore.CMFCorePermissions import ManagePortal
-from Products.CMFCore.CMFCorePermissions import ModifyPortalContent
 from Products.CMFCore.ActionProviderBase import ActionProviderBase
-from utils import _dtmldir
-
 from Products.CMFCore.interfaces.portal_metadata \
         import portal_metadata as IMetadataTool
+from Products.CMFCore.utils import UniqueObject
+
+from exceptions import MetadataError
+from permissions import ManagePortal
+from permissions import ModifyPortalContent
+from permissions import View
+from utils import _dtmldir
 
 
 class MetadataElementPolicy( SimpleItem ):
@@ -46,8 +49,8 @@ class MetadataElementPolicy( SimpleItem ):
     enforce_vocabulary  = 0
     allowed_vocabulary  = ()
 
-    def __init__( self, is_multi_valued=0 ):
-        self.is_multi_valued    = not not is_multi_valued
+    def __init__(self, is_multi_valued=False):
+        self.is_multi_valued = bool(is_multi_valued)
 
     #
     #   Mutator.
@@ -60,11 +63,11 @@ class MetadataElementPolicy( SimpleItem ):
             , enforce_vocabulary
             , allowed_vocabulary
             ):
-        self.is_required        = not not is_required
-        self.supply_default     = not not supply_default
+        self.is_required        = bool(is_required)
+        self.supply_default     = bool(supply_default)
         self.default_value      = default_value
-        self.enforce_vocabulary = not not enforce_vocabulary
-        self.allowed_vocabulary = tuple( allowed_vocabulary )
+        self.enforce_vocabulary = bool(enforce_vocabulary)
+        self.allowed_vocabulary = tuple(allowed_vocabulary)
 
     #
     #   Query interface
@@ -120,6 +123,7 @@ DEFAULT_ELEMENT_SPECS = ( ( 'Title', 0 )
                         , ( 'Rights', 0 )
                         )
 
+
 class ElementSpec( SimpleItem ):
     """
         Represent all the tool knows about a single metadata element.
@@ -136,7 +140,6 @@ class ElementSpec( SimpleItem ):
         self.policies         = PersistentMapping()
         self.policies[ None ] = self._makePolicy()  # set default policy
 
-
     security.declarePrivate( '_makePolicy' )
     def _makePolicy( self ):
         return MetadataElementPolicy( self.is_multi_valued )
@@ -157,7 +160,7 @@ class ElementSpec( SimpleItem ):
         try:
             return self.policies[ typ ].__of__(self)
         except KeyError:
-            return self.policies[ None ]
+            return self.policies[ None ].__of__(self)
 
     security.declareProtected(View , 'listPolicies')
     def listPolicies( self ):
@@ -195,8 +198,6 @@ class ElementSpec( SimpleItem ):
 
 InitializeClass( ElementSpec )
 
-class MetadataError( Exception ):
-    pass
 
 class MetadataTool( UniqueObject, SimpleItem, ActionProviderBase ):
 
@@ -401,7 +402,7 @@ class MetadataTool( UniqueObject, SimpleItem, ActionProviderBase ):
         """
         # Don't replace.
         if self.element_specs.has_key( element ):
-            return
+           return
 
         self.element_specs[ element ] = ElementSpec( is_multi_valued )
 
@@ -535,12 +536,12 @@ class MetadataTool( UniqueObject, SimpleItem, ActionProviderBase ):
                 raise MetadataError, \
                         'Metadata element %s is required.' % element
 
-            if policy.enforceVocabulary():
+            if value and policy.enforceVocabulary():
                 values = policy.isMultiValued() and value or [ value ]
                 for value in values:
                     if not value in policy.allowedVocabulary():
                         raise MetadataError, \
-                        'Value %s is not in allowed vocabulary for' \
+                        'Value %s is not in allowed vocabulary for ' \
                         'metadata element %s.' % ( value, element )
 
         # TODO:  Call validation_hook, if present
