@@ -3,16 +3,16 @@
 # Copyright (c) 2003 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """ Extended membership tool interface.
 
-$Id: portal_membership.py,v 1.2 2003/02/21 11:38:39 yuppie Exp $
+$Id: portal_membership.py 36457 2004-08-12 15:07:44Z jens $
 """
 
 from Products.CMFCore.interfaces.portal_membership \
@@ -32,14 +32,3 @@ class portal_membership(BaseInterface):
 
         Permission -- Manage portal
         """
-
-    def getMembersFolder():
-        """ Get the members folder object.
-
-        If no members folder is set or the set folder id doesn't exist, None
-        is returned.
-
-        Permission -- Always available
-
-        Returns -- Members folder object or None
-        """
