@@ -1,48 +1,134 @@
+##############################################################################
+#
+# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+#
+# This software is subject to the provisions of the Zope Public License,
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
+# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
+# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
+# FOR A PARTICULAR PURPOSE.
+#
+##############################################################################
+""" Unit tests for Document module.
+
+$Id: test_Document.py 38418 2005-09-09 08:40:13Z yuppie $
+"""
+
 from unittest import TestSuite, makeSuite, main
-
+import Testing
 try:
-    from Interface.Verify import verifyClass
-except ImportError:
-    # for Zope versions before 2.6.0
-    from Interface import verify_class_implementation as verifyClass
+    import Zope2
+except ImportError: # BBB: for Zope 2.7
+    import Zope as Zope2
+Zope2.startup()
 
-from StringIO import StringIO
+from os.path import abspath
+from os.path import dirname
+from os.path import join as path_join
 from re import compile
+from StringIO import StringIO
 
-from Products.CMFCore.tests.base.testcase import RequestTest
+from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
+from DocumentTemplate.DT_Util import html_quote
 
-from Products.CMFCore.tests.base.content import DOCTYPE
-from Products.CMFCore.tests.base.content import HTML_TEMPLATE
 from Products.CMFCore.tests.base.content import BASIC_HTML
-from Products.CMFCore.tests.base.content import FAUX_HTML_LEADING_TEXT
-from Products.CMFCore.tests.base.content import ENTITY_IN_TITLE
 from Products.CMFCore.tests.base.content import BASIC_STRUCTUREDTEXT
-from Products.CMFCore.tests.base.content import STX_WITH_HTML
+from Products.CMFCore.tests.base.content import DOCTYPE
+from Products.CMFCore.tests.base.content import ENTITY_IN_TITLE
+from Products.CMFCore.tests.base.content import FAUX_HTML_LEADING_TEXT
+from Products.CMFCore.tests.base.content import HTML_TEMPLATE
+from Products.CMFCore.tests.base.content import SIMPLE_HTML
+from Products.CMFCore.tests.base.content import SIMPLE_STRUCTUREDTEXT
+from Products.CMFCore.tests.base.content import SIMPLE_XHTML
 from Products.CMFCore.tests.base.content import STX_NO_HEADERS
 from Products.CMFCore.tests.base.content import STX_NO_HEADERS_BUT_COLON
-from Products.CMFCore.tests.base.content import SIMPLE_STRUCTUREDTEXT
-from Products.CMFCore.tests.base.content import SIMPLE_HTML
-from Products.CMFCore.tests.base.content import SIMPLE_XHTML
+from Products.CMFCore.tests.base.content import STX_WITH_HTML
+from Products.CMFCore.tests.base.dummy import DummySite
+from Products.CMFCore.tests.base.dummy import DummyTool
+from Products.CMFCore.tests.base.testcase import RequestTest
+from Products.CMFCore.tests.base.tidata import FTIDATA_CMF15
+from Products.CMFCore.TypesTool import FactoryTypeInformation as FTI
+from Products.CMFCore.TypesTool import TypesTool
+from Products.CMFDefault import utils
 
-from Products.CMFDefault.Document import Document
 
-class DocumentTests(RequestTest):
+class RequestTestBase(RequestTest):
 
     def setUp(self):
         RequestTest.setUp(self)
-        self.d = Document('foo')
+        self.site = DummySite('site').__of__(self.root)
+        self.site._setObject( 'portal_membership', DummyTool() )
+
+    def _makeOne(self, id, *args, **kw):
+        from Products.CMFDefault.Document import Document
+
+        return self.site._setObject( id, Document(id, *args, **kw) )
+
+
+class DocumentTests(RequestTestBase):
+
+    def test_z2interfaces(self):
+        from Interface.Verify import verifyClass
+        from Products.CMFCore.interfaces.Contentish \
+                import Contentish as IContentish
+        from Products.CMFCore.interfaces.DublinCore \
+                import CatalogableDublinCore as ICatalogableDublinCore
+        from Products.CMFCore.interfaces.DublinCore \
+                import DublinCore as IDublinCore
+        from Products.CMFCore.interfaces.DublinCore \
+                import MutableDublinCore as IMutableDublinCore
+        from Products.CMFCore.interfaces.Dynamic \
+                import DynamicType as IDynamicType
+        from Products.CMFDefault.Document import Document
+        from Products.CMFDefault.interfaces.Document import IDocument
+        from Products.CMFDefault.interfaces.Document import IMutableDocument
+
+        verifyClass(ICatalogableDublinCore, Document)
+        verifyClass(IContentish, Document)
+        verifyClass(IDocument, Document)
+        verifyClass(IDublinCore, Document)
+        verifyClass(IDynamicType, Document)
+        verifyClass(IMutableDocument, Document)
+        verifyClass(IMutableDublinCore, Document)
+
+    def test_z3interfaces(self):
+        try:
+            from zope.interface.verify import verifyClass
+            from Products.CMFCore.interfaces import ICatalogableDublinCore
+            from Products.CMFCore.interfaces import IContentish
+            from Products.CMFCore.interfaces import IDublinCore
+            from Products.CMFCore.interfaces import IDynamicType
+            from Products.CMFCore.interfaces import IMutableDublinCore
+            from Products.CMFDefault.interfaces import IDocument
+            from Products.CMFDefault.interfaces import IMutableDocument
+        except ImportError:
+            # BBB: for Zope 2.7
+            return
+        from Products.CMFDefault.Document import Document
+
+        verifyClass(ICatalogableDublinCore, Document)
+        verifyClass(IContentish, Document)
+        verifyClass(IDocument, Document)
+        verifyClass(IDublinCore, Document)
+        verifyClass(IDynamicType, Document)
+        verifyClass(IMutableDocument, Document)
+        verifyClass(IMutableDublinCore, Document)
 
     def test_Empty(self):
-        d = Document('foo', text_format='structured-text')
+        d = self._makeOne('foo', text_format='structured-text')
+
         self.assertEqual( d.title, '' )
         self.assertEqual( d.description, '' )
         self.assertEqual( d.text, '' )
         self.assertEqual( d.text_format, 'structured-text' )
         self.assertEqual( d._stx_level, 1 )
+        self.assertEqual( d.get_size(), 0 )
 
     def test_editBasicHTML(self):
-        d = self.d
+        d = self._makeOne('foo')
         d.edit('html', BASIC_HTML)
+
         self.failUnless( hasattr(d, 'cooked_text') )
         self.assertEqual( d.Format(), 'text/html' )
         self.assertEqual( d.text.find('</body>'), -1 )
@@ -54,16 +140,18 @@ class DocumentTests(RequestTest):
         self.assertEqual( d._stx_level, 1 )
 
     def test_editSimpleXHTML(self):
-        d = self.d
+        d = self._makeOne('foo')
         d.edit('html', SIMPLE_XHTML)
+
         self.failUnless( hasattr(d, 'cooked_text') )
         self.assertEqual( d.Format(), 'text/html' )
         self.assertEqual( d.cooked_text, '\n  <h1>Not a lot here</h1>\n ')
 
     def test_UpperedHtml(self):
         self.REQUEST['BODY'] = BASIC_HTML.upper()
-        d = self.d
+        d = self._makeOne('foo')
         d.PUT(self.REQUEST, self.RESPONSE)
+
         self.assertEqual( d.Format(), 'text/html' )
         self.assertEqual( d.title, 'TITLE IN TAG' )
         self.assertEqual( d.text.find('</BODY'), -1 )
@@ -72,25 +160,28 @@ class DocumentTests(RequestTest):
 
     def test_EntityInTitle(self):
         self.REQUEST['BODY'] = ENTITY_IN_TITLE
-        d = self.d
+        d = self._makeOne('foo')
         d.PUT(self.REQUEST, self.RESPONSE)
+
         self.assertEqual( d.title, '&Auuml;rger' )
 
     def test_HtmlWithDoctype(self):
-        d = self.d
         self.REQUEST['BODY'] = '%s\n%s' % (DOCTYPE, BASIC_HTML)
+        d = self._makeOne('foo')
         d.PUT(self.REQUEST, self.RESPONSE)
+
         self.assertEqual( d.Description(), 'Describe me' )
 
     def test_HtmlWithoutNewlines(self):
-        d = self.d
         self.REQUEST['BODY'] = ''.join((BASIC_HTML.split('\n')))
+        d = self._makeOne('foo')
         d.PUT(self.REQUEST, self.RESPONSE)
+
         self.assertEqual( d.Format(), 'text/html' )
         self.assertEqual( d.Description(), 'Describe me' )
 
     def test_EditPlainDocumentWithEmbeddedHTML(self):
-        d = self.d
+        d = self._makeOne('foo')
         d.edit('structured-text', FAUX_HTML_LEADING_TEXT)
         fully_edited = d.cooked_text
         d._edit(FAUX_HTML_LEADING_TEXT, 'structured-text')
@@ -98,7 +189,7 @@ class DocumentTests(RequestTest):
         self.assertEquals(fully_edited, partly_edited)
 
     def test_BigHtml(self):
-        d = self.d
+        d = self._makeOne('foo')
         s = []
         looper = '<li> number %s</li>'
         for i in range(12000): s.append(looper % i)
@@ -109,33 +200,65 @@ class DocumentTests(RequestTest):
         self.assertEqual( d.CookedBody(), body )
 
     def test_BigHtml_via_upload(self):
-        d = self.d
+        d = self._makeOne('foo')
         s = []
         looper = '<li> number %s</li>'
         for i in range(12000): s.append(looper % i)
         body = '<ul>\n%s\n</ul>' % '\n'.join(s)
         html = HTML_TEMPLATE % {'title': 'big document',
                                 'body': body}
-        file = StringIO( html )
-        d.edit(text_format='html', text='', file=file)
+        _file = StringIO( html )
+        d.edit(text_format='html', text='', file=_file)
         self.assertEqual( d.CookedBody(), body )
 
+    def test_Htmltag_removal_and_formatchange(self):
+        # Test for http://www.zope.org/Collectors/CMF/214
+        d = self._makeOne('foo')
+        quoted_html = html_quote(BASIC_HTML)
+
+        # Put HTML into a plain text document
+        d.edit(text_format='plain', text=BASIC_HTML)
+        new_body = d.CookedBody()
+        self.failIf(new_body==BASIC_HTML)
+        self.failUnless(new_body.startswith(quoted_html[:5]))
+
+        # Now we change the format. The body *must* change because
+        # the format change will trigger re-cooking
+        old_body = d.CookedBody()
+        d.setFormat('html')
+        new_body = d.CookedBody()
+        self.failIf(old_body==new_body)
+        self.failIf(new_body==BASIC_HTML)
+
+    def test_Html_Fragment(self):
+        # Test that edits with HTML fragments behave nicely
+        FRAGMENT = '<div id="placeholder">CONTENT</div>'
+        d = self._makeOne('foo')
+        d.edit(text_format='html', text=FRAGMENT)
+        self.assertEqual( d.CookedBody(), FRAGMENT )
+        self.assertEqual( d.get_size(), len(FRAGMENT) )
+
     def test_plain_text(self):
-        """test that plain text forrmat works"""
-        d = self.d
-        d.edit(text_format='plain', text='*some plain text*\nwith a newline')
-        self.assertEqual( d.CookedBody(), '*some plain text*<br>with a newline')
+        # test that plain text forrmat works
+        PLAIN_TEXT = '*some plain text*\nwith a newline'
+        d = self._makeOne('foo')
+        d.edit(text_format='plain', text=PLAIN_TEXT)
+        self.assertEqual( d.CookedBody(),
+                          '*some plain text*<br />with a newline')
+        self.assertEqual( d.get_size(), len(PLAIN_TEXT) )
 
     def test_EditStructuredTextWithHTML(self):
-        d = self.d
+        d = self._makeOne('foo')
         d.edit(text_format='structured-text', text=STX_WITH_HTML)
 
         self.assertEqual( d.Format(), 'text/plain' )
+        self.assertEqual( d.get_size(), len(STX_WITH_HTML) )
 
     def test_StructuredText(self):
         self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
-        d = self.d
+        d = self._makeOne('foo')
         d.PUT(self.REQUEST, self.RESPONSE)
+
         self.failUnless( hasattr(d, 'cooked_text') )
         self.assertEqual( d.Format(), 'text/plain' )
         self.assertEqual( d.Title(), 'My Document' )
@@ -159,7 +282,7 @@ class DocumentTests(RequestTest):
                                 ] )
 
     def test_STX_Levels(self):
-        d = self.d
+        d = self._makeOne('foo')
         d.edit(text_format='structured-text', text=BASIC_STRUCTUREDTEXT)
         self.assertEqual( d._stx_level, 1 )
 
@@ -184,7 +307,7 @@ class DocumentTests(RequestTest):
 
     def test_Init(self):
         self.REQUEST['BODY']=BASIC_STRUCTUREDTEXT
-        d = self.d
+        d = self._makeOne('foo')
         d.PUT(self.REQUEST, self.RESPONSE)
         self.assertEqual( d.Format(), 'text/plain' )
         self.assertEqual( d.Title(), 'My Document' )
@@ -192,28 +315,30 @@ class DocumentTests(RequestTest):
         self.assertEqual( len(d.Contributors()), 3 )
         self.failUnless( d.cooked_text.find('<p>') >= 0 )
 
-        d = Document('foo', text='')
+        d = self._makeOne('foo', text='')
         self.REQUEST['BODY']=BASIC_HTML
         d.PUT(self.REQUEST, self.RESPONSE)
         self.assertEqual( d.Format(), 'text/html' )
         self.assertEqual( d.Title(), 'Title in tag' )
         self.assertEqual( len(d.Contributors()), 3 )
 
-        d = Document('foo', text_format='structured-text', title='Foodoc')
+        d = self._makeOne('foo', text_format='structured-text',
+                          title='Foodoc')
         self.assertEqual( d.text, '' )
         self.failIf( d.CookedBody() )
         self.assertEqual( d.title, 'Foodoc' )
         self.assertEqual( d.Format(), 'text/plain' )
 
         # Tracker issue 435:  initial text is not cooked.
-        d = Document('foo', text_format='structured-text', text=STX_NO_HEADERS)
+        d = self._makeOne('foo', text_format='structured-text',
+                          text=STX_NO_HEADERS)
         self.assertEqual( d.EditableBody(), STX_NO_HEADERS )
         self.failUnless( d.CookedBody() )
         self.assertEqual( d.Format(), 'text/plain' )
 
     def test_STX_NoHeaders( self ):
         self.REQUEST['BODY']=STX_NO_HEADERS
-        d = self.d
+        d = self._makeOne('foo')
         d.editMetadata( title="Plain STX"
                        , description="Look, Ma, no headers!"
                        , subject=( "plain", "STX" )
@@ -235,7 +360,7 @@ class DocumentTests(RequestTest):
         self.failUnless( 'STX' in d.Subject() )
 
     def test_STX_NoHeaders_but_colon( self ):
-        d = self.d
+        d = self._makeOne('foo')
         d.editMetadata( title="Plain STX"
                        , description="Look, Ma, no headers!"
                        , subject=( "plain", "STX" )
@@ -245,7 +370,7 @@ class DocumentTests(RequestTest):
         self.assertEqual( d.EditableBody(), STX_NO_HEADERS_BUT_COLON )
 
     def test_ZMI_edit( self ):
-        d = self.d
+        d = self._makeOne('foo')
         d.editMetadata( title="Plain STX"
                        , description="Look, Ma, no headers!"
                        , subject=( "plain", "STX" )
@@ -256,7 +381,7 @@ class DocumentTests(RequestTest):
         self.assertEqual( d.EditableBody(), STX_NO_HEADERS_BUT_COLON )
 
     def test_Format_methods(self):
-        d = self.d
+        d = self._makeOne('foo')
         d.setFormat('plain')
         self.assertEqual( d.text_format, 'plain' )
         self.assertEqual( d.Format(), 'text/plain' )
@@ -278,31 +403,32 @@ class DocumentTests(RequestTest):
         d.setFormat('foo')
         self.assertEqual( d.text_format, 'structured-text' )
 
-    def test_interface(self):
-        from Products.CMFCore.interfaces.Dynamic \
-                import DynamicType as IDynamicType
-        from Products.CMFCore.interfaces.Contentish \
-                import Contentish as IContentish
-        from Products.CMFCore.interfaces.DublinCore \
-                import DublinCore as IDublinCore
-        from Products.CMFCore.interfaces.DublinCore \
-                import CatalogableDublinCore as ICatalogableDublinCore
-        from Products.CMFCore.interfaces.DublinCore \
-                import MutableDublinCore as IMutableDublinCore
+    def test_default_format( self ):
+        d = self._makeOne('foo', text='')
 
-        verifyClass(IDynamicType, Document)
-        verifyClass(IContentish, Document)
-        verifyClass(IDublinCore, Document)
-        verifyClass(ICatalogableDublinCore, Document)
-        # XXX: verifyClass doesn't work with WorkflowMethod wrappers.
-        # verifyClass(IMutableDublinCore, Document)
+        self.assertEqual( d.Format(), 'text/plain' )
+        self.assertEqual( d.text_format, 'structured-text' )
 
 
-class DocumentFTPGetTests(RequestTest):
+class DocumentFTPGetTests(RequestTestBase):
 
     def testHTML( self ):
         self.REQUEST['BODY']=BASIC_HTML
-        d = Document( 'foo' )
+
+        ttool = self.site._setObject( 'portal_types', TypesTool() )
+        fti = FTIDATA_CMF15[0].copy()
+        del fti['id']
+        ttool._setObject( 'Document', FTI('Document', **fti) )
+
+        zpt = self.site._setObject( 'source_html',
+                                    ZopePageTemplate('source_html') )
+        dir = abspath( dirname(utils.__file__) )
+        _file = path_join(dir, 'skins', 'zpt_content', 'source_html.pt')
+        data = open(_file, 'r').read()
+        zpt.write(data)
+
+        d = self._makeOne('foo')
+        d._setPortalTypeName('Document')
         d.PUT(self.REQUEST, self.RESPONSE)
 
         rnlinesplit = compile( r'\r?\n?' )
@@ -347,9 +473,14 @@ class DocumentFTPGetTests(RequestTest):
         for header in simple_headers:
             self.failUnless( header in get_headers )
 
+        body1 = d.EditableBody()
+        self.REQUEST['BODY'] = d.manage_FTPget()
+        d.PUT(self.REQUEST, self.RESPONSE)
+        self.assertEqual( d.EditableBody(), body1 )
+
     def testSTX( self ):
         self.REQUEST['BODY']=SIMPLE_STRUCTUREDTEXT
-        d = Document( 'foo' )
+        d = self._makeOne('foo')
         d.PUT(self.REQUEST, self.RESPONSE)
 
         rnlinesplit = compile( r'\r?\n?' )
@@ -377,14 +508,17 @@ class DocumentFTPGetTests(RequestTest):
 
 class DocumentPUTTests(RequestTest):
 
-    def setUp(self):
-        RequestTest.setUp(self)
-        self.d = Document('foo')
+    def _makeOne(self, id, *args, **kw):
+        from Products.CMFDefault.Document import Document
+
+        # NullResource.PUT calls the PUT method on the bare object!
+        return Document(id, *args, **kw)
 
     def test_PUTBasicHTML(self):
         self.REQUEST['BODY'] = BASIC_HTML
-        d = self.d
+        d = self._makeOne('foo')
         r = d.PUT(self.REQUEST, self.RESPONSE)
+
         self.failUnless( hasattr(d, 'cooked_text') )
         self.assertEqual( d.Format(), 'text/html' )
         self.assertEqual( d.title, 'Title in tag' )
@@ -407,8 +541,9 @@ class DocumentPUTTests(RequestTest):
 
     def test_PUTSimpleXHTML(self):
         self.REQUEST['BODY'] = SIMPLE_XHTML
-        d = self.d
+        d = self._makeOne('foo')
         r = d.PUT(self.REQUEST, self.RESPONSE)
+
         self.failUnless( hasattr(d, 'cooked_text') )
         self.assertEqual( d.Format(), 'text/html' )
         self.assertEqual( d.Description(), 'Describe me' )
@@ -416,35 +551,37 @@ class DocumentPUTTests(RequestTest):
         self.assertEqual( r.status, 204 )
 
     def test_PutStructuredTextWithHTML(self):
-
         self.REQUEST['BODY'] = STX_WITH_HTML
+        d = self._makeOne('foo')
+        r = d.PUT(self.REQUEST, self.RESPONSE)
 
-        r = self.d.PUT(self.REQUEST, self.RESPONSE)
-        self.assertEqual( self.d.Format(), 'text/plain' )
+        self.assertEqual( d.Format(), 'text/plain' )
         self.assertEqual( r.status, 204 )
 
     def test_PutStructuredText(self):
-
         self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
+        d = self._makeOne('foo')
+        r = d.PUT(self.REQUEST, self.RESPONSE)
 
-        r = self.d.PUT(self.REQUEST, self.RESPONSE)
-        self.assertEqual( self.d.Format(), 'text/plain' )
+        self.assertEqual( d.Format(), 'text/plain' )
         self.assertEqual( r.status, 204 )
 
     def test_PutHtmlWithDoctype(self):
-
         html = '%s\n\n  \n   %s' % (DOCTYPE, BASIC_HTML)
         self.REQUEST['BODY'] = html
+        d = self._makeOne('foo')
+        r = d.PUT(self.REQUEST, self.RESPONSE)
 
-        r = self.d.PUT(self.REQUEST, self.RESPONSE)
-        self.assertEqual( self.d.Format(), 'text/html' )
-        self.assertEqual( self.d.Description(), 'Describe me' )
+        self.assertEqual( d.Format(), 'text/html' )
+        self.assertEqual( d.Description(), 'Describe me' )
         self.assertEqual( r.status, 204 )
 
     def test_PutHtmlWithoutMetadata(self):
-        self.REQUEST['BODY'] = HTML_TEMPLATE % {'title': 'Foo', 'body': 'Bar'}
-        d = self.d
+        html = HTML_TEMPLATE % {'title': 'Foo', 'body': 'Bar'}
+        self.REQUEST['BODY'] = html
+        d = self._makeOne('foo')
         r = d.PUT(self.REQUEST, self.RESPONSE)
+
         self.assertEqual( d.Title(), 'Foo' )
         self.assertEqual( d.Format(), 'text/html' )
         self.assertEqual( d.Description(), '' )
