@@ -3,48 +3,51 @@
 # Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """ Web-configurable workflow.
 
-$Id: DCWorkflow.py,v 1.24.4.2 2004/04/22 17:47:58 sidnei Exp $
+$Id: DCWorkflow.py 36993 2005-04-26 16:01:34Z anguenot $
 """
 
-# Python library
-from string import join
-
 # Zope
-from AccessControl import getSecurityManager, ClassSecurityInfo
+from AccessControl import ClassSecurityInfo
+from AccessControl import getSecurityManager
+from AccessControl import Unauthorized
+from Acquisition import aq_inner
+from Acquisition import aq_parent
+from App.Undo import UndoSupport
+from DocumentTemplate.DT_Util import TemplateDict
+from Globals import InitializeClass
 from OFS.Folder import Folder
 from OFS.ObjectManager import bad_id
-from OFS.Traversable import Traversable
-from Globals import DTMLFile
-import Acquisition
-from Acquisition import aq_inner, aq_parent
-import Globals
-import App
-from DocumentTemplate.DT_Util import TemplateDict
 
 # CMFCore
-from Products.CMFCore.WorkflowCore import WorkflowException, \
-     ObjectDeleted, ObjectMoved
-from Products.CMFCore.WorkflowTool import addWorkflowFactory
-from Products.CMFCore.CMFCorePermissions import ManagePortal
+from Products.CMFCore.interfaces.portal_workflow \
+        import WorkflowDefinition as IWorkflowDefinition
 from Products.CMFCore.utils import getToolByName
+from Products.CMFCore.WorkflowCore import ObjectDeleted
+from Products.CMFCore.WorkflowCore import ObjectMoved
+from Products.CMFCore.WorkflowCore import WorkflowException
+from Products.CMFCore.WorkflowTool import addWorkflowFactory
 
 # DCWorkflow
-from utils import _dtmldir, modifyRolesForPermission
+from permissions import ManagePortal
+from utils import _dtmldir
+from utils import modifyRolesForPermission
+from utils import modifyRolesForGroup
 from WorkflowUIMixin import WorkflowUIMixin
-from Transitions import TRIGGER_AUTOMATIC, TRIGGER_USER_ACTION, \
-     TRIGGER_WORKFLOW_METHOD
-from Expression import StateChangeInfo, createExprContext
+from Transitions import TRIGGER_AUTOMATIC
+from Transitions import TRIGGER_USER_ACTION
+from Transitions import TRIGGER_WORKFLOW_METHOD
+from Expression import StateChangeInfo
+from Expression import createExprContext
 
-Unauthorized = 'Unauthorized'
 
 def checkId(id):
     res = bad_id(id)
@@ -59,6 +62,9 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
     workflow definition.
     UI methods are in WorkflowUIMixin.
     '''
+
+    __implements__ = IWorkflowDefinition
+
     meta_type = 'Workflow'
     title = 'DC Workflow Definition'
     _isAWorkflow = 1
@@ -73,6 +79,13 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
     scripts = None
 
     permissions = ()
+    groups = ()     # Names of groups managed by this workflow.
+    roles = None  # The role names managed by this workflow.
+    # If roles is None, listRoles() provides a default.
+
+    creation_guard = None  # The guard that can veto object creation.
+
+    manager_bypass = 0  # Boolean: 'Manager' role bypasses guards
 
     manage_options = (
         {'label': 'Properties', 'action': 'manage_properties'},
@@ -82,7 +95,8 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
         {'label': 'Worklists', 'action': 'worklists/manage_main'},
         {'label': 'Scripts', 'action': 'scripts/manage_main'},
         {'label': 'Permissions', 'action': 'manage_permissions'},
-        ) + App.Undo.UndoSupport.manage_options
+        {'label': 'Groups', 'action': 'manage_groups'},
+        )
 
     security = ClassSecurityInfo()
     security.declareObjectProtected(ManagePortal)
@@ -170,10 +184,10 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
         Allows this workflow to
         include actions to be displayed in the actions box.
         Called only when this workflow is applicable to
-        info.content.
+        info.object.
         Returns the actions to be displayed to the user.
         '''
-        ob = info.content
+        ob = info.object
         sdef = self._getWorkflowStateOf(ob)
         if sdef is None:
             return None
@@ -191,7 +205,7 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
                             'category': tdef.actbox_category,
                             'transition': tdef}))
         res.sort()
-        return map((lambda (id, val): val), res)
+        return [ result[1] for result in res ]
 
     security.declarePrivate('listGlobalActions')
     def listGlobalActions(self, info):
@@ -216,29 +230,27 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
                     if var_match_keys:
                         # Check the catalog for items in the worklist.
                         catalog = getToolByName(self, 'portal_catalog')
-                        dict = {}
+                        kw = {}
                         for k in var_match_keys:
                             v = qdef.getVarMatch(k)
-                            v_fmt = map(lambda x, info=info: x%info, v)
-                            dict[k] = v_fmt
-                        searchres = apply(catalog.searchResults, (), dict)
+                            kw[k] = [ x % info for x in v ]
+                        searchres = catalog.searchResults(**kw)
                         if not searchres:
                             continue
                     if fmt_data is None:
                         fmt_data = TemplateDict()
                         fmt_data._push(info)
-                    searchres_len = lambda searchres=searchres: len(searchres)
-                    fmt_data._push({'count': searchres_len})
+                    fmt_data._push({'count': len(searchres)})
                     res.append((id, {'name': qdef.actbox_name % fmt_data,
                                      'url': qdef.actbox_url % fmt_data,
                                      'permissions': (),  # Predetermined.
                                      'category': qdef.actbox_category}))
                     fmt_data._pop()
         res.sort()
-        return map((lambda (id, val): val), res)
+        return [ result[1] for result in res ]
 
     security.declarePrivate('isActionSupported')
-    def isActionSupported(self, ob, action):
+    def isActionSupported(self, ob, action, **kw):
         '''
         Returns a true value if the given action name
         is possible in the current state.
@@ -250,27 +262,28 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
             tdef = self.transitions.get(action, None)
             if (tdef is not None and
                 tdef.trigger_type == TRIGGER_USER_ACTION and
-                self._checkTransitionGuard(tdef, ob)):
+                self._checkTransitionGuard(tdef, ob, **kw)):
                 return 1
         return 0
 
     security.declarePrivate('doActionFor')
-    def doActionFor(self, ob, action, **kw):
+    def doActionFor(self, ob, action, comment='', **kw):
         '''
         Allows the user to request a workflow action.  This method
         must perform its own security checks.
         '''
+        kw['comment'] = comment
         sdef = self._getWorkflowStateOf(ob)
         if sdef is None:
             raise WorkflowException, 'Object is in an undefined state'
         if action not in sdef.transitions:
-            raise Unauthorized
+            raise Unauthorized(action)
         tdef = self.transitions.get(action, None)
         if tdef is None or tdef.trigger_type != TRIGGER_USER_ACTION:
             raise WorkflowException, (
                 'Transition %s is not triggered by a user action' % action)
-        if not self._checkTransitionGuard(tdef, ob):
-            raise Unauthorized
+        if not self._checkTransitionGuard(tdef, ob, **kw):
+            raise Unauthorized(action)
         self._changeStateOf(ob, tdef, kw)
 
     security.declarePrivate('isWorkflowMethodSupported')
@@ -300,15 +313,15 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
         if sdef is None:
             raise WorkflowException, 'Object is in an undefined state'
         if method_id not in sdef.transitions:
-            raise Unauthorized
+            raise Unauthorized(method_id)
         tdef = self.transitions.get(method_id, None)
         if tdef is None or tdef.trigger_type != TRIGGER_WORKFLOW_METHOD:
             raise WorkflowException, (
                 'Transition %s is not triggered by a workflow method'
                 % method_id)
         if not self._checkTransitionGuard(tdef, ob):
-            raise Unauthorized
-        res = apply(func, args, kw)
+            raise Unauthorized(method_id)
+        res = func(*args, **kw)
         try:
             self._changeStateOf(ob, tdef)
         except ObjectDeleted:
@@ -356,12 +369,21 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
 
         return value
 
+    security.declarePrivate('allowCreate')
+    def allowCreate(self, container, type_name):
+        """Returns true if the user is allowed to create a workflow instance.
+
+        The object passed to the guard is the prospective container.
+        """
+        if self.creation_guard is not None:
+            return self.creation_guard.check(
+                getSecurityManager(), self, container)
+        return 1
+
     security.declarePrivate('notifyCreated')
     def notifyCreated(self, ob):
-        '''
-        Notifies this workflow after an object has been created
-        and put in its new place.
-        '''
+        """Notifies this workflow after an object has been created and added.
+        """
         try:
             self._changeStateOf(ob, None)
         except ( ObjectDeleted, ObjectMoved ):
@@ -394,26 +416,37 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
 
     security.declarePrivate('updateRoleMappingsFor')
     def updateRoleMappingsFor(self, ob):
-        '''
-        Changes the object permissions according to the current
-        state.
-        '''
+        """Changes the object permissions according to the current state.
+        """
         changed = 0
         sdef = self._getWorkflowStateOf(ob)
-        if sdef is not None and self.permissions:
+        if sdef is None:
+            return 0
+        # Update the role -> permission map.
+        if self.permissions:
             for p in self.permissions:
                 roles = []
                 if sdef.permission_roles is not None:
                     roles = sdef.permission_roles.get(p, roles)
                 if modifyRolesForPermission(ob, p, roles):
                     changed = 1
+        # Update the group -> role map.
+        groups = self.getGroups()
+        managed_roles = self.getRoles()
+        if groups and managed_roles:
+            for group in groups:
+                roles = ()
+                if sdef.group_roles is not None:
+                    roles = sdef.group_roles.get(group, ())
+                if modifyRolesForGroup(ob, group, roles, managed_roles):
+                    changed = 1
         return changed
 
-    def _checkTransitionGuard(self, t, ob):
+    def _checkTransitionGuard(self, t, ob, **kw):
         guard = t.guard
         if guard is None:
             return 1
-        if guard.check(getSecurityManager(), self, ob):
+        if guard.check(getSecurityManager(), self, ob, **kw):
             return 1
         return 0
 
@@ -426,7 +459,7 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
                     tdef = t
                     break
         return tdef
-
+        
     def _changeStateOf(self, ob, tdef=None, kwargs=None):
         '''
         Changes state.  Can execute multiple transitions if there are
@@ -548,8 +581,8 @@ class DCWorkflowDefinition (WorkflowUIMixin, Folder):
         else:
             return new_sdef
 
+InitializeClass(DCWorkflowDefinition)
 
-Globals.InitializeClass(DCWorkflowDefinition)
 
 addWorkflowFactory(DCWorkflowDefinition, id='dc_workflow',
                    title='Web-configurable workflow')
